/*
qsampler - A LinuxSampler Qt GUI Interface.

   Copyright (C) 2004-2020, rncbc aka Rui Nuno Capela. All rights reserved.
   Copyright (C) 2007, Christian Schoenebeck

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


*/

/********************************************************************************
** Form generated from reading UI file 'qsamplerDeviceForm.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_QSAMPLERDEVICEFORM_H
#define UI_QSAMPLERDEVICEFORM_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSplitter>
#include <QtWidgets/QTableView>
#include <QtWidgets/QTreeWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_qsamplerDeviceForm
{
public:
    QVBoxLayout *vboxLayout;
    QSplitter *splitter1;
    QTreeWidget *DeviceListView;
    QSplitter *splitter2;
    QWidget *DeviceParamLayout;
    QVBoxLayout *vboxLayout1;
    QHBoxLayout *hboxLayout;
    QLabel *DeviceNameTextLabel;
    QLabel *DriverNameTextLabel;
    QComboBox *DriverNameComboBox;
    QTableView *DeviceParamTable;
    QWidget *PortParamLayout;
    QVBoxLayout *vboxLayout2;
    QHBoxLayout *hboxLayout1;
    QLabel *DevicePortTextLabel;
    QComboBox *DevicePortComboBox;
    QTableView *DevicePortParamTable;
    QHBoxLayout *hboxLayout2;
    QPushButton *RefreshDevicesPushButton;
    QSpacerItem *spacerItem;
    QPushButton *CreateDevicePushButton;
    QPushButton *DeleteDevicePushButton;
    QPushButton *ClosePushButton;

    void setupUi(QDialog *qsamplerDeviceForm)
    {
        if (qsamplerDeviceForm->objectName().isEmpty())
            qsamplerDeviceForm->setObjectName("qsamplerDeviceForm");
        qsamplerDeviceForm->resize(601, 378);
        qsamplerDeviceForm->setFocusPolicy(Qt::StrongFocus);
        const QIcon icon = QIcon(QString::fromUtf8(":/images/qsamplerDevice.png"));
        qsamplerDeviceForm->setWindowIcon(icon);
        vboxLayout = new QVBoxLayout(qsamplerDeviceForm);
        vboxLayout->setSpacing(6);
        vboxLayout->setContentsMargins(9, 9, 9, 9);
        vboxLayout->setObjectName("vboxLayout");
        splitter1 = new QSplitter(qsamplerDeviceForm);
        splitter1->setObjectName("splitter1");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(splitter1->sizePolicy().hasHeightForWidth());
        splitter1->setSizePolicy(sizePolicy);
        splitter1->setOrientation(Qt::Horizontal);
        splitter1->setHandleWidth(4);
        DeviceListView = new QTreeWidget(splitter1);
        DeviceListView->setObjectName("DeviceListView");
        DeviceListView->setMinimumSize(QSize(120, 0));
        DeviceListView->setRootIsDecorated(false);
        DeviceListView->setUniformRowHeights(true);
        splitter1->addWidget(DeviceListView);
        splitter2 = new QSplitter(splitter1);
        splitter2->setObjectName("splitter2");
        sizePolicy.setHeightForWidth(splitter2->sizePolicy().hasHeightForWidth());
        splitter2->setSizePolicy(sizePolicy);
        splitter2->setOrientation(Qt::Vertical);
        splitter2->setHandleWidth(4);
        DeviceParamLayout = new QWidget(splitter2);
        DeviceParamLayout->setObjectName("DeviceParamLayout");
        vboxLayout1 = new QVBoxLayout(DeviceParamLayout);
        vboxLayout1->setSpacing(6);
        vboxLayout1->setContentsMargins(0, 0, 0, 0);
        vboxLayout1->setObjectName("vboxLayout1");
        vboxLayout1->setContentsMargins(0, 0, 0, 0);
        hboxLayout = new QHBoxLayout();
        hboxLayout->setSpacing(6);
        hboxLayout->setContentsMargins(0, 0, 0, 0);
        hboxLayout->setObjectName("hboxLayout");
        DeviceNameTextLabel = new QLabel(DeviceParamLayout);
        DeviceNameTextLabel->setObjectName("DeviceNameTextLabel");
        QFont font;
        font.setBold(true);
        DeviceNameTextLabel->setFont(font);
        DeviceNameTextLabel->setWordWrap(false);
        DeviceNameTextLabel->setIndent(4);

        hboxLayout->addWidget(DeviceNameTextLabel);

        DriverNameTextLabel = new QLabel(DeviceParamLayout);
        DriverNameTextLabel->setObjectName("DriverNameTextLabel");
        DriverNameTextLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        DriverNameTextLabel->setWordWrap(false);

        hboxLayout->addWidget(DriverNameTextLabel);

        DriverNameComboBox = new QComboBox(DeviceParamLayout);
        DriverNameComboBox->setObjectName("DriverNameComboBox");

        hboxLayout->addWidget(DriverNameComboBox);


        vboxLayout1->addLayout(hboxLayout);

        DeviceParamTable = new QTableView(DeviceParamLayout);
        DeviceParamTable->setObjectName("DeviceParamTable");
        DeviceParamTable->setAlternatingRowColors(true);
        DeviceParamTable->setSelectionMode(QAbstractItemView::NoSelection);
        DeviceParamTable->setShowGrid(false);

        vboxLayout1->addWidget(DeviceParamTable);

        splitter2->addWidget(DeviceParamLayout);
        PortParamLayout = new QWidget(splitter2);
        PortParamLayout->setObjectName("PortParamLayout");
        vboxLayout2 = new QVBoxLayout(PortParamLayout);
        vboxLayout2->setSpacing(6);
        vboxLayout2->setContentsMargins(0, 0, 0, 0);
        vboxLayout2->setObjectName("vboxLayout2");
        vboxLayout2->setContentsMargins(0, 0, 0, 0);
        hboxLayout1 = new QHBoxLayout();
        hboxLayout1->setSpacing(6);
        hboxLayout1->setContentsMargins(0, 0, 0, 0);
        hboxLayout1->setObjectName("hboxLayout1");
        DevicePortTextLabel = new QLabel(PortParamLayout);
        DevicePortTextLabel->setObjectName("DevicePortTextLabel");
        DevicePortTextLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        DevicePortTextLabel->setWordWrap(false);

        hboxLayout1->addWidget(DevicePortTextLabel);

        DevicePortComboBox = new QComboBox(PortParamLayout);
        DevicePortComboBox->setObjectName("DevicePortComboBox");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(DevicePortComboBox->sizePolicy().hasHeightForWidth());
        DevicePortComboBox->setSizePolicy(sizePolicy1);
        DevicePortComboBox->setMinimumSize(QSize(120, 0));

        hboxLayout1->addWidget(DevicePortComboBox);


        vboxLayout2->addLayout(hboxLayout1);

        DevicePortParamTable = new QTableView(PortParamLayout);
        DevicePortParamTable->setObjectName("DevicePortParamTable");
        DevicePortParamTable->setAlternatingRowColors(true);
        DevicePortParamTable->setSelectionMode(QAbstractItemView::NoSelection);
        DevicePortParamTable->setShowGrid(false);

        vboxLayout2->addWidget(DevicePortParamTable);

        splitter2->addWidget(PortParamLayout);
        splitter1->addWidget(splitter2);

        vboxLayout->addWidget(splitter1);

        hboxLayout2 = new QHBoxLayout();
        hboxLayout2->setSpacing(4);
        hboxLayout2->setContentsMargins(0, 0, 0, 0);
        hboxLayout2->setObjectName("hboxLayout2");
        RefreshDevicesPushButton = new QPushButton(qsamplerDeviceForm);
        RefreshDevicesPushButton->setObjectName("RefreshDevicesPushButton");
        const QIcon icon1 = QIcon(QString::fromUtf8(":/images/formRefresh.png"));
        RefreshDevicesPushButton->setIcon(icon1);

        hboxLayout2->addWidget(RefreshDevicesPushButton);

        spacerItem = new QSpacerItem(180, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout2->addItem(spacerItem);

        CreateDevicePushButton = new QPushButton(qsamplerDeviceForm);
        CreateDevicePushButton->setObjectName("CreateDevicePushButton");
        const QIcon icon2 = QIcon(QString::fromUtf8(":/images/deviceCreate.png"));
        CreateDevicePushButton->setIcon(icon2);

        hboxLayout2->addWidget(CreateDevicePushButton);

        DeleteDevicePushButton = new QPushButton(qsamplerDeviceForm);
        DeleteDevicePushButton->setObjectName("DeleteDevicePushButton");
        const QIcon icon3 = QIcon(QString::fromUtf8(":/images/deviceDelete.png"));
        DeleteDevicePushButton->setIcon(icon3);

        hboxLayout2->addWidget(DeleteDevicePushButton);

        ClosePushButton = new QPushButton(qsamplerDeviceForm);
        ClosePushButton->setObjectName("ClosePushButton");
        const QIcon icon4 = QIcon(QString::fromUtf8(":/images/formReject.png"));
        ClosePushButton->setIcon(icon4);

        hboxLayout2->addWidget(ClosePushButton);


        vboxLayout->addLayout(hboxLayout2);

#if QT_CONFIG(shortcut)
        DriverNameTextLabel->setBuddy(DriverNameComboBox);
        DevicePortTextLabel->setBuddy(DevicePortComboBox);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(DeviceListView, RefreshDevicesPushButton);
        QWidget::setTabOrder(RefreshDevicesPushButton, DriverNameComboBox);
        QWidget::setTabOrder(DriverNameComboBox, DevicePortComboBox);
        QWidget::setTabOrder(DevicePortComboBox, CreateDevicePushButton);
        QWidget::setTabOrder(CreateDevicePushButton, DeleteDevicePushButton);
        QWidget::setTabOrder(DeleteDevicePushButton, ClosePushButton);

        retranslateUi(qsamplerDeviceForm);

        QMetaObject::connectSlotsByName(qsamplerDeviceForm);
    } // setupUi

    void retranslateUi(QDialog *qsamplerDeviceForm)
    {
        qsamplerDeviceForm->setWindowTitle(QCoreApplication::translate("qsamplerDeviceForm", "Devices", nullptr));
        QTreeWidgetItem *___qtreewidgetitem = DeviceListView->headerItem();
        ___qtreewidgetitem->setText(0, QCoreApplication::translate("qsamplerDeviceForm", "Devices", nullptr));
#if QT_CONFIG(tooltip)
        DeviceListView->setToolTip(QCoreApplication::translate("qsamplerDeviceForm", "Device list", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        DeviceNameTextLabel->setToolTip(QCoreApplication::translate("qsamplerDeviceForm", "Device name", nullptr));
#endif // QT_CONFIG(tooltip)
        DriverNameTextLabel->setText(QCoreApplication::translate("qsamplerDeviceForm", "Dri&ver:", nullptr));
#if QT_CONFIG(tooltip)
        DriverNameComboBox->setToolTip(QCoreApplication::translate("qsamplerDeviceForm", "Driver type name", nullptr));
#endif // QT_CONFIG(tooltip)
        DevicePortTextLabel->setText(QCoreApplication::translate("qsamplerDeviceForm", "Channel:", nullptr));
#if QT_CONFIG(tooltip)
        DevicePortComboBox->setToolTip(QCoreApplication::translate("qsamplerDeviceForm", "Device port/channel", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        RefreshDevicesPushButton->setToolTip(QCoreApplication::translate("qsamplerDeviceForm", "Refresh device list view", nullptr));
#endif // QT_CONFIG(tooltip)
        RefreshDevicesPushButton->setText(QCoreApplication::translate("qsamplerDeviceForm", "&Refresh", nullptr));
#if QT_CONFIG(tooltip)
        CreateDevicePushButton->setToolTip(QCoreApplication::translate("qsamplerDeviceForm", "Create device", nullptr));
#endif // QT_CONFIG(tooltip)
        CreateDevicePushButton->setText(QCoreApplication::translate("qsamplerDeviceForm", "&Create", nullptr));
#if QT_CONFIG(tooltip)
        DeleteDevicePushButton->setToolTip(QCoreApplication::translate("qsamplerDeviceForm", "Delete device", nullptr));
#endif // QT_CONFIG(tooltip)
        DeleteDevicePushButton->setText(QCoreApplication::translate("qsamplerDeviceForm", "&Delete", nullptr));
#if QT_CONFIG(tooltip)
        ClosePushButton->setToolTip(QCoreApplication::translate("qsamplerDeviceForm", "Close this dialog", nullptr));
#endif // QT_CONFIG(tooltip)
        ClosePushButton->setText(QCoreApplication::translate("qsamplerDeviceForm", "Close", nullptr));
    } // retranslateUi

};

namespace Ui {
    class qsamplerDeviceForm: public Ui_qsamplerDeviceForm {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_QSAMPLERDEVICEFORM_H
