/*
qsampler - A LinuxSampler Qt GUI Interface.

   Copyright (C) 2004-2020, rncbc aka Rui Nuno Capela. All rights reserved.
   Copyright (C) 2007, Christian Schoenebeck

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


*/

/********************************************************************************
** Form generated from reading UI file 'qsamplerInstrumentListForm.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_QSAMPLERINSTRUMENTLISTFORM_H
#define UI_QSAMPLERINSTRUMENTLISTFORM_H

#include <QtCore/QVariant>
#include <QtGui/QAction>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QMainWindow>
#include <QtWidgets/QToolBar>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_qsamplerInstrumentListForm
{
public:
    QAction *newInstrumentAction;
    QAction *editInstrumentAction;
    QAction *deleteInstrumentAction;
    QAction *refreshInstrumentsAction;
    QWidget *centralWidget;
    QToolBar *instrumentToolbar;

    void setupUi(QMainWindow *qsamplerInstrumentListForm)
    {
        if (qsamplerInstrumentListForm->objectName().isEmpty())
            qsamplerInstrumentListForm->setObjectName("qsamplerInstrumentListForm");
        qsamplerInstrumentListForm->resize(720, 340);
        const QIcon icon = QIcon(QString::fromUtf8(":/images/qsamplerInstrument.png"));
        qsamplerInstrumentListForm->setWindowIcon(icon);
        newInstrumentAction = new QAction(qsamplerInstrumentListForm);
        newInstrumentAction->setObjectName("newInstrumentAction");
        const QIcon icon1 = QIcon(QString::fromUtf8(":/images/itemNew.png"));
        newInstrumentAction->setIcon(icon1);
        editInstrumentAction = new QAction(qsamplerInstrumentListForm);
        editInstrumentAction->setObjectName("editInstrumentAction");
        const QIcon icon2 = QIcon(QString::fromUtf8(":/images/formEdit.png"));
        editInstrumentAction->setIcon(icon2);
        deleteInstrumentAction = new QAction(qsamplerInstrumentListForm);
        deleteInstrumentAction->setObjectName("deleteInstrumentAction");
        const QIcon icon3 = QIcon(QString::fromUtf8(":/images/formRemove.png"));
        deleteInstrumentAction->setIcon(icon3);
        refreshInstrumentsAction = new QAction(qsamplerInstrumentListForm);
        refreshInstrumentsAction->setObjectName("refreshInstrumentsAction");
        const QIcon icon4 = QIcon(QString::fromUtf8(":/images/formRefresh.png"));
        refreshInstrumentsAction->setIcon(icon4);
        centralWidget = new QWidget(qsamplerInstrumentListForm);
        centralWidget->setObjectName("centralWidget");
        qsamplerInstrumentListForm->setCentralWidget(centralWidget);
        instrumentToolbar = new QToolBar(qsamplerInstrumentListForm);
        instrumentToolbar->setObjectName("instrumentToolbar");
        instrumentToolbar->setOrientation(Qt::Horizontal);
        qsamplerInstrumentListForm->addToolBar(Qt::ToolBarArea::TopToolBarArea, instrumentToolbar);

        retranslateUi(qsamplerInstrumentListForm);

        QMetaObject::connectSlotsByName(qsamplerInstrumentListForm);
    } // setupUi

    void retranslateUi(QMainWindow *qsamplerInstrumentListForm)
    {
        qsamplerInstrumentListForm->setWindowTitle(QCoreApplication::translate("qsamplerInstrumentListForm", "Instruments", nullptr));
        newInstrumentAction->setText(QCoreApplication::translate("qsamplerInstrumentListForm", "New &Instrument...", nullptr));
        newInstrumentAction->setIconText(QCoreApplication::translate("qsamplerInstrumentListForm", "New", nullptr));
#if QT_CONFIG(shortcut)
        newInstrumentAction->setShortcut(QCoreApplication::translate("qsamplerInstrumentListForm", "Ins", nullptr));
#endif // QT_CONFIG(shortcut)
        editInstrumentAction->setText(QCoreApplication::translate("qsamplerInstrumentListForm", "&Edit...", nullptr));
        editInstrumentAction->setIconText(QCoreApplication::translate("qsamplerInstrumentListForm", "Edit", nullptr));
#if QT_CONFIG(shortcut)
        editInstrumentAction->setShortcut(QCoreApplication::translate("qsamplerInstrumentListForm", "Enter", nullptr));
#endif // QT_CONFIG(shortcut)
        deleteInstrumentAction->setText(QCoreApplication::translate("qsamplerInstrumentListForm", "&Delete", nullptr));
        deleteInstrumentAction->setIconText(QCoreApplication::translate("qsamplerInstrumentListForm", "Delete", nullptr));
#if QT_CONFIG(shortcut)
        deleteInstrumentAction->setShortcut(QCoreApplication::translate("qsamplerInstrumentListForm", "Del", nullptr));
#endif // QT_CONFIG(shortcut)
        refreshInstrumentsAction->setText(QCoreApplication::translate("qsamplerInstrumentListForm", "&Refresh", nullptr));
        refreshInstrumentsAction->setIconText(QCoreApplication::translate("qsamplerInstrumentListForm", "Refresh", nullptr));
#if QT_CONFIG(shortcut)
        refreshInstrumentsAction->setShortcut(QCoreApplication::translate("qsamplerInstrumentListForm", "F5", nullptr));
#endif // QT_CONFIG(shortcut)
    } // retranslateUi

};

namespace Ui {
    class qsamplerInstrumentListForm: public Ui_qsamplerInstrumentListForm {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_QSAMPLERINSTRUMENTLISTFORM_H
