/*
qsampler - A LinuxSampler Qt GUI Interface.

   Copyright (C) 2004-2021, rncbc aka Rui Nuno Capela. All rights reserved.
   Copyright (C) 2007, 2008, 2014 Christian Schoenebeck

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


*/

/********************************************************************************
** Form generated from reading UI file 'qsamplerChannelStrip.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_QSAMPLERCHANNELSTRIP_H
#define UI_QSAMPLERCHANNELSTRIP_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QProgressBar>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSlider>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_qsamplerChannelStrip
{
public:
    QHBoxLayout *hboxLayout;
    QPushButton *ChannelSetupPushButton;
    QFrame *ChannelInfoFrame;
    QGridLayout *gridLayout;
    QLabel *EngineNameTextLabel;
    QLabel *MidiPortChannelTextLabel;
    QLabel *InstrumentStatusTextLabel;
    QLabel *MidiActivityLabel;
    QPushButton *InstrumentNamePushButton;
    QPushButton *ChannelMutePushButton;
    QPushButton *ChannelSoloPushButton;
    QSlider *ChannelVolumeSlider;
    QSpinBox *ChannelVolumeSpinBox;
    QPushButton *ChannelFxPushButton;
    QPushButton *ChannelEditPushButton;
    QSpacerItem *spacerItem;
    QProgressBar *StreamUsageProgressBar;
    QLabel *StreamVoiceCountTextLabel;
    QSpacerItem *spacerItem1;

    void setupUi(QWidget *qsamplerChannelStrip)
    {
        if (qsamplerChannelStrip->objectName().isEmpty())
            qsamplerChannelStrip->setObjectName("qsamplerChannelStrip");
        qsamplerChannelStrip->resize(640, 64);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(qsamplerChannelStrip->sizePolicy().hasHeightForWidth());
        qsamplerChannelStrip->setSizePolicy(sizePolicy);
        qsamplerChannelStrip->setFocusPolicy(Qt::StrongFocus);
        const QIcon icon = QIcon(QString::fromUtf8(":/images/qsamplerChannel.png"));
        qsamplerChannelStrip->setWindowIcon(icon);
        hboxLayout = new QHBoxLayout(qsamplerChannelStrip);
        hboxLayout->setSpacing(4);
        hboxLayout->setContentsMargins(4, 4, 4, 4);
        hboxLayout->setObjectName("hboxLayout");
        ChannelSetupPushButton = new QPushButton(qsamplerChannelStrip);
        ChannelSetupPushButton->setObjectName("ChannelSetupPushButton");
        ChannelSetupPushButton->setMinimumSize(QSize(100, 22));
        ChannelSetupPushButton->setMaximumSize(QSize(120, 32));
        ChannelSetupPushButton->setIcon(icon);

        hboxLayout->addWidget(ChannelSetupPushButton);

        ChannelInfoFrame = new QFrame(qsamplerChannelStrip);
        ChannelInfoFrame->setObjectName("ChannelInfoFrame");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(ChannelInfoFrame->sizePolicy().hasHeightForWidth());
        ChannelInfoFrame->setSizePolicy(sizePolicy1);
        ChannelInfoFrame->setMinimumSize(QSize(240, 48));
        ChannelInfoFrame->setMaximumSize(QSize(320, 64));
        ChannelInfoFrame->setAutoFillBackground(true);
        ChannelInfoFrame->setFrameShape(QFrame::StyledPanel);
        ChannelInfoFrame->setFrameShadow(QFrame::Sunken);
        gridLayout = new QGridLayout(ChannelInfoFrame);
        gridLayout->setSpacing(0);
        gridLayout->setContentsMargins(2, 2, 2, 2);
        gridLayout->setObjectName("gridLayout");
        EngineNameTextLabel = new QLabel(ChannelInfoFrame);
        EngineNameTextLabel->setObjectName("EngineNameTextLabel");

        gridLayout->addWidget(EngineNameTextLabel, 0, 0, 1, 1);

        MidiPortChannelTextLabel = new QLabel(ChannelInfoFrame);
        MidiPortChannelTextLabel->setObjectName("MidiPortChannelTextLabel");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Preferred);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(MidiPortChannelTextLabel->sizePolicy().hasHeightForWidth());
        MidiPortChannelTextLabel->setSizePolicy(sizePolicy2);
        MidiPortChannelTextLabel->setMinimumSize(QSize(20, 0));
        MidiPortChannelTextLabel->setAlignment(Qt::AlignCenter);
        MidiPortChannelTextLabel->setWordWrap(false);

        gridLayout->addWidget(MidiPortChannelTextLabel, 0, 2, 1, 1);

        InstrumentStatusTextLabel = new QLabel(ChannelInfoFrame);
        InstrumentStatusTextLabel->setObjectName("InstrumentStatusTextLabel");
        InstrumentStatusTextLabel->setMinimumSize(QSize(20, 0));
        InstrumentStatusTextLabel->setAlignment(Qt::AlignCenter);
        InstrumentStatusTextLabel->setWordWrap(false);

        gridLayout->addWidget(InstrumentStatusTextLabel, 1, 2, 1, 1);

        MidiActivityLabel = new QLabel(ChannelInfoFrame);
        MidiActivityLabel->setObjectName("MidiActivityLabel");
        QSizePolicy sizePolicy3(QSizePolicy::Policy::Maximum, QSizePolicy::Policy::Maximum);
        sizePolicy3.setHorizontalStretch(0);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(MidiActivityLabel->sizePolicy().hasHeightForWidth());
        MidiActivityLabel->setSizePolicy(sizePolicy3);
        MidiActivityLabel->setMinimumSize(QSize(16, 16));
        MidiActivityLabel->setMaximumSize(QSize(16, 16));
        MidiActivityLabel->setBaseSize(QSize(16, 16));
        MidiActivityLabel->setFrameShape(QFrame::NoFrame);
        MidiActivityLabel->setFrameShadow(QFrame::Plain);
        MidiActivityLabel->setAlignment(Qt::AlignCenter);

        gridLayout->addWidget(MidiActivityLabel, 0, 1, 1, 1);

        InstrumentNamePushButton = new QPushButton(ChannelInfoFrame);
        InstrumentNamePushButton->setObjectName("InstrumentNamePushButton");
        InstrumentNamePushButton->setStyleSheet(QString::fromUtf8("text-align: left; margin: 0px;"));
        InstrumentNamePushButton->setFlat(true);

        gridLayout->addWidget(InstrumentNamePushButton, 1, 0, 1, 1);


        hboxLayout->addWidget(ChannelInfoFrame);

        ChannelMutePushButton = new QPushButton(qsamplerChannelStrip);
        ChannelMutePushButton->setObjectName("ChannelMutePushButton");
        ChannelMutePushButton->setMaximumSize(QSize(48, 32));
        ChannelMutePushButton->setCheckable(true);

        hboxLayout->addWidget(ChannelMutePushButton);

        ChannelSoloPushButton = new QPushButton(qsamplerChannelStrip);
        ChannelSoloPushButton->setObjectName("ChannelSoloPushButton");
        ChannelSoloPushButton->setMaximumSize(QSize(48, 32));
        ChannelSoloPushButton->setCheckable(true);

        hboxLayout->addWidget(ChannelSoloPushButton);

        ChannelVolumeSlider = new QSlider(qsamplerChannelStrip);
        ChannelVolumeSlider->setObjectName("ChannelVolumeSlider");
        ChannelVolumeSlider->setMinimumSize(QSize(100, 0));
        ChannelVolumeSlider->setMaximum(100);
        ChannelVolumeSlider->setOrientation(Qt::Horizontal);
        ChannelVolumeSlider->setTickPosition(QSlider::TicksBothSides);
        ChannelVolumeSlider->setTickInterval(10);

        hboxLayout->addWidget(ChannelVolumeSlider);

        ChannelVolumeSpinBox = new QSpinBox(qsamplerChannelStrip);
        ChannelVolumeSpinBox->setObjectName("ChannelVolumeSpinBox");
        ChannelVolumeSpinBox->setMinimumSize(QSize(60, 0));
        ChannelVolumeSpinBox->setMaximumSize(QSize(120, 24));
        ChannelVolumeSpinBox->setMaximum(100);

        hboxLayout->addWidget(ChannelVolumeSpinBox);

        ChannelFxPushButton = new QPushButton(qsamplerChannelStrip);
        ChannelFxPushButton->setObjectName("ChannelFxPushButton");
        ChannelFxPushButton->setMaximumSize(QSize(48, 32));

        hboxLayout->addWidget(ChannelFxPushButton);

        ChannelEditPushButton = new QPushButton(qsamplerChannelStrip);
        ChannelEditPushButton->setObjectName("ChannelEditPushButton");
        ChannelEditPushButton->setMaximumSize(QSize(48, 32));

        hboxLayout->addWidget(ChannelEditPushButton);

        spacerItem = new QSpacerItem(8, 22, QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Minimum);

        hboxLayout->addItem(spacerItem);

        StreamUsageProgressBar = new QProgressBar(qsamplerChannelStrip);
        StreamUsageProgressBar->setObjectName("StreamUsageProgressBar");
        QSizePolicy sizePolicy4(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Fixed);
        sizePolicy4.setHorizontalStretch(0);
        sizePolicy4.setVerticalStretch(0);
        sizePolicy4.setHeightForWidth(StreamUsageProgressBar->sizePolicy().hasHeightForWidth());
        StreamUsageProgressBar->setSizePolicy(sizePolicy4);
        StreamUsageProgressBar->setMinimumSize(QSize(64, 22));
        StreamUsageProgressBar->setOrientation(Qt::Horizontal);

        hboxLayout->addWidget(StreamUsageProgressBar);

        StreamVoiceCountTextLabel = new QLabel(qsamplerChannelStrip);
        StreamVoiceCountTextLabel->setObjectName("StreamVoiceCountTextLabel");
        StreamVoiceCountTextLabel->setMinimumSize(QSize(48, 22));
        StreamVoiceCountTextLabel->setMaximumSize(QSize(64, 32));
        StreamVoiceCountTextLabel->setAutoFillBackground(true);
        StreamVoiceCountTextLabel->setFrameShape(QFrame::StyledPanel);
        StreamVoiceCountTextLabel->setFrameShadow(QFrame::Sunken);
        StreamVoiceCountTextLabel->setAlignment(Qt::AlignCenter);
        StreamVoiceCountTextLabel->setWordWrap(false);

        hboxLayout->addWidget(StreamVoiceCountTextLabel);

        spacerItem1 = new QSpacerItem(8, 22, QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Minimum);

        hboxLayout->addItem(spacerItem1);

        QWidget::setTabOrder(ChannelSetupPushButton, ChannelMutePushButton);
        QWidget::setTabOrder(ChannelMutePushButton, ChannelSoloPushButton);
        QWidget::setTabOrder(ChannelSoloPushButton, ChannelVolumeSlider);
        QWidget::setTabOrder(ChannelVolumeSlider, ChannelVolumeSpinBox);
        QWidget::setTabOrder(ChannelVolumeSpinBox, ChannelFxPushButton);
        QWidget::setTabOrder(ChannelFxPushButton, ChannelEditPushButton);

        retranslateUi(qsamplerChannelStrip);

        QMetaObject::connectSlotsByName(qsamplerChannelStrip);
    } // setupUi

    void retranslateUi(QWidget *qsamplerChannelStrip)
    {
        qsamplerChannelStrip->setWindowTitle(QCoreApplication::translate("qsamplerChannelStrip", "Channel", nullptr));
#if QT_CONFIG(tooltip)
        ChannelSetupPushButton->setToolTip(QCoreApplication::translate("qsamplerChannelStrip", "Channel setup", nullptr));
#endif // QT_CONFIG(tooltip)
        ChannelSetupPushButton->setText(QCoreApplication::translate("qsamplerChannelStrip", "&Channel", nullptr));
        EngineNameTextLabel->setText(QCoreApplication::translate("qsamplerChannelStrip", "--", nullptr));
#if QT_CONFIG(tooltip)
        MidiPortChannelTextLabel->setToolTip(QCoreApplication::translate("qsamplerChannelStrip", "MIDI port / channel", nullptr));
#endif // QT_CONFIG(tooltip)
        MidiPortChannelTextLabel->setText(QCoreApplication::translate("qsamplerChannelStrip", "-- / --", nullptr));
#if QT_CONFIG(tooltip)
        InstrumentStatusTextLabel->setToolTip(QCoreApplication::translate("qsamplerChannelStrip", "Instrument load status", nullptr));
#endif // QT_CONFIG(tooltip)
        InstrumentStatusTextLabel->setText(QCoreApplication::translate("qsamplerChannelStrip", "--", nullptr));
#if QT_CONFIG(tooltip)
        MidiActivityLabel->setToolTip(QCoreApplication::translate("qsamplerChannelStrip", "MIDI activity", nullptr));
#endif // QT_CONFIG(tooltip)
        MidiActivityLabel->setText(QString());
#if QT_CONFIG(tooltip)
        InstrumentNamePushButton->setToolTip(QCoreApplication::translate("qsamplerChannelStrip", "Instrument name", nullptr));
#endif // QT_CONFIG(tooltip)
        InstrumentNamePushButton->setText(QCoreApplication::translate("qsamplerChannelStrip", "--", nullptr));
#if QT_CONFIG(tooltip)
        ChannelMutePushButton->setToolTip(QCoreApplication::translate("qsamplerChannelStrip", "Channel mute", nullptr));
#endif // QT_CONFIG(tooltip)
        ChannelMutePushButton->setText(QCoreApplication::translate("qsamplerChannelStrip", "&Mute", nullptr));
#if QT_CONFIG(tooltip)
        ChannelSoloPushButton->setToolTip(QCoreApplication::translate("qsamplerChannelStrip", "Channel solo", nullptr));
#endif // QT_CONFIG(tooltip)
        ChannelSoloPushButton->setText(QCoreApplication::translate("qsamplerChannelStrip", "&Solo", nullptr));
#if QT_CONFIG(tooltip)
        ChannelVolumeSlider->setToolTip(QCoreApplication::translate("qsamplerChannelStrip", "Channel volume", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        ChannelVolumeSpinBox->setToolTip(QCoreApplication::translate("qsamplerChannelStrip", "Channel volume", nullptr));
#endif // QT_CONFIG(tooltip)
        ChannelVolumeSpinBox->setSuffix(QCoreApplication::translate("qsamplerChannelStrip", " %", nullptr));
#if QT_CONFIG(tooltip)
        ChannelFxPushButton->setToolTip(QCoreApplication::translate("qsamplerChannelStrip", "Edit Channel's Effect Settings", nullptr));
#endif // QT_CONFIG(tooltip)
        ChannelFxPushButton->setText(QCoreApplication::translate("qsamplerChannelStrip", "&FX", nullptr));
#if QT_CONFIG(tooltip)
        ChannelEditPushButton->setToolTip(QCoreApplication::translate("qsamplerChannelStrip", "Edit channel's instrument", nullptr));
#endif // QT_CONFIG(tooltip)
        ChannelEditPushButton->setText(QCoreApplication::translate("qsamplerChannelStrip", "&Edit", nullptr));
#if QT_CONFIG(tooltip)
        StreamUsageProgressBar->setToolTip(QCoreApplication::translate("qsamplerChannelStrip", "Least buffer fill stream usage (%)", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        StreamVoiceCountTextLabel->setToolTip(QCoreApplication::translate("qsamplerChannelStrip", "Stream / Voice count", nullptr));
#endif // QT_CONFIG(tooltip)
        StreamVoiceCountTextLabel->setText(QCoreApplication::translate("qsamplerChannelStrip", "--/--", nullptr));
    } // retranslateUi

};

namespace Ui {
    class qsamplerChannelStrip: public Ui_qsamplerChannelStrip {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_QSAMPLERCHANNELSTRIP_H
