/*
qsampler - A LinuxSampler Qt GUI Interface.

   Copyright (C) 2004-2025, rncbc aka Rui Nuno Capela. All rights reserved.
   Copyright (C) 2007-2019 Christian Schoenebeck

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


*/

/********************************************************************************
** Form generated from reading UI file 'qsamplerMainForm.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_QSAMPLERMAINFORM_H
#define UI_QSAMPLERMAINFORM_H

#include <QtCore/QVariant>
#include <QtGui/QAction>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QMainWindow>
#include <QtWidgets/QMenu>
#include <QtWidgets/QMenuBar>
#include <QtWidgets/QStatusBar>
#include <QtWidgets/QToolBar>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_qsamplerMainForm
{
public:
    QAction *fileNewAction;
    QAction *fileOpenAction;
    QAction *fileSaveAction;
    QAction *fileSaveAsAction;
    QAction *fileResetAction;
    QAction *fileRestartAction;
    QAction *fileExitAction;
    QAction *editAddChannelAction;
    QAction *editRemoveChannelAction;
    QAction *editResetChannelAction;
    QAction *editResetAllChannelsAction;
    QAction *editSetupChannelAction;
    QAction *editEditChannelAction;
    QAction *viewMenubarAction;
    QAction *viewToolbarAction;
    QAction *viewStatusbarAction;
    QAction *viewMessagesAction;
    QAction *viewInstrumentsAction;
    QAction *viewDevicesAction;
    QAction *viewOptionsAction;
    QAction *channelsArrangeAction;
    QAction *channelsAutoArrangeAction;
    QAction *helpAboutAction;
    QAction *helpAboutQtAction;
    QWidget *centralwidget;
    QToolBar *fileToolbar;
    QToolBar *editToolbar;
    QToolBar *channelsToolbar;
    QMenuBar *MenuBar;
    QMenu *editMenu;
    QMenu *viewMenu;
    QMenu *viewMidiDeviceStatusMenu;
    QMenu *channelsMenu;
    QMenu *helpMenu;
    QMenu *fileMenu;
    QMenu *fileOpenRecentMenu;
    QStatusBar *statusbar;

    void setupUi(QMainWindow *qsamplerMainForm)
    {
        if (qsamplerMainForm->objectName().isEmpty())
            qsamplerMainForm->setObjectName("qsamplerMainForm");
        qsamplerMainForm->resize(800, 600);
        const QIcon icon = QIcon(QString::fromUtf8(":/images/qsampler.svg"));
        qsamplerMainForm->setWindowIcon(icon);
        fileNewAction = new QAction(qsamplerMainForm);
        fileNewAction->setObjectName("fileNewAction");
        const QIcon icon1 = QIcon(QString::fromUtf8(":/images/fileNew.png"));
        fileNewAction->setIcon(icon1);
        fileOpenAction = new QAction(qsamplerMainForm);
        fileOpenAction->setObjectName("fileOpenAction");
        const QIcon icon2 = QIcon(QString::fromUtf8(":/images/fileOpen.png"));
        fileOpenAction->setIcon(icon2);
        fileSaveAction = new QAction(qsamplerMainForm);
        fileSaveAction->setObjectName("fileSaveAction");
        const QIcon icon3 = QIcon(QString::fromUtf8(":/images/fileSave.png"));
        fileSaveAction->setIcon(icon3);
        fileSaveAsAction = new QAction(qsamplerMainForm);
        fileSaveAsAction->setObjectName("fileSaveAsAction");
        fileResetAction = new QAction(qsamplerMainForm);
        fileResetAction->setObjectName("fileResetAction");
        const QIcon icon4 = QIcon(QString::fromUtf8(":/images/fileReset.png"));
        fileResetAction->setIcon(icon4);
        fileRestartAction = new QAction(qsamplerMainForm);
        fileRestartAction->setObjectName("fileRestartAction");
        const QIcon icon5 = QIcon(QString::fromUtf8(":/images/fileRestart.png"));
        fileRestartAction->setIcon(icon5);
        fileExitAction = new QAction(qsamplerMainForm);
        fileExitAction->setObjectName("fileExitAction");
        editAddChannelAction = new QAction(qsamplerMainForm);
        editAddChannelAction->setObjectName("editAddChannelAction");
        const QIcon icon6 = QIcon(QString::fromUtf8(":/images/editAddChannel.png"));
        editAddChannelAction->setIcon(icon6);
        editRemoveChannelAction = new QAction(qsamplerMainForm);
        editRemoveChannelAction->setObjectName("editRemoveChannelAction");
        const QIcon icon7 = QIcon(QString::fromUtf8(":/images/editRemoveChannel.png"));
        editRemoveChannelAction->setIcon(icon7);
        editResetChannelAction = new QAction(qsamplerMainForm);
        editResetChannelAction->setObjectName("editResetChannelAction");
        const QIcon icon8 = QIcon(QString::fromUtf8(":/images/editResetChannel.png"));
        editResetChannelAction->setIcon(icon8);
        editResetAllChannelsAction = new QAction(qsamplerMainForm);
        editResetAllChannelsAction->setObjectName("editResetAllChannelsAction");
        const QIcon icon9 = QIcon(QString::fromUtf8(":/images/editResetAllChannels.png"));
        editResetAllChannelsAction->setIcon(icon9);
        editSetupChannelAction = new QAction(qsamplerMainForm);
        editSetupChannelAction->setObjectName("editSetupChannelAction");
        const QIcon icon10 = QIcon(QString::fromUtf8(":/images/editSetupChannel.png"));
        editSetupChannelAction->setIcon(icon10);
        editEditChannelAction = new QAction(qsamplerMainForm);
        editEditChannelAction->setObjectName("editEditChannelAction");
        const QIcon icon11 = QIcon(QString::fromUtf8(":/images/editEditChannel.png"));
        editEditChannelAction->setIcon(icon11);
        viewMenubarAction = new QAction(qsamplerMainForm);
        viewMenubarAction->setObjectName("viewMenubarAction");
        viewMenubarAction->setCheckable(true);
        viewToolbarAction = new QAction(qsamplerMainForm);
        viewToolbarAction->setObjectName("viewToolbarAction");
        viewToolbarAction->setCheckable(true);
        viewStatusbarAction = new QAction(qsamplerMainForm);
        viewStatusbarAction->setObjectName("viewStatusbarAction");
        viewStatusbarAction->setCheckable(true);
        viewMessagesAction = new QAction(qsamplerMainForm);
        viewMessagesAction->setObjectName("viewMessagesAction");
        viewMessagesAction->setCheckable(true);
        viewInstrumentsAction = new QAction(qsamplerMainForm);
        viewInstrumentsAction->setObjectName("viewInstrumentsAction");
        viewInstrumentsAction->setCheckable(true);
        const QIcon icon12 = QIcon(QString::fromUtf8(":/images/qsamplerInstrument.png"));
        viewInstrumentsAction->setIcon(icon12);
        viewDevicesAction = new QAction(qsamplerMainForm);
        viewDevicesAction->setObjectName("viewDevicesAction");
        viewDevicesAction->setCheckable(true);
        const QIcon icon13 = QIcon(QString::fromUtf8(":/images/qsamplerDevice.png"));
        viewDevicesAction->setIcon(icon13);
        viewOptionsAction = new QAction(qsamplerMainForm);
        viewOptionsAction->setObjectName("viewOptionsAction");
        channelsArrangeAction = new QAction(qsamplerMainForm);
        channelsArrangeAction->setObjectName("channelsArrangeAction");
        channelsArrangeAction->setCheckable(false);
        const QIcon icon14 = QIcon(QString::fromUtf8(":/images/channelsArrange.png"));
        channelsArrangeAction->setIcon(icon14);
        channelsAutoArrangeAction = new QAction(qsamplerMainForm);
        channelsAutoArrangeAction->setObjectName("channelsAutoArrangeAction");
        channelsAutoArrangeAction->setCheckable(true);
        helpAboutAction = new QAction(qsamplerMainForm);
        helpAboutAction->setObjectName("helpAboutAction");
        helpAboutQtAction = new QAction(qsamplerMainForm);
        helpAboutQtAction->setObjectName("helpAboutQtAction");
        centralwidget = new QWidget(qsamplerMainForm);
        centralwidget->setObjectName("centralwidget");
        qsamplerMainForm->setCentralWidget(centralwidget);
        fileToolbar = new QToolBar(qsamplerMainForm);
        fileToolbar->setObjectName("fileToolbar");
        fileToolbar->setOrientation(Qt::Horizontal);
        qsamplerMainForm->addToolBar(Qt::ToolBarArea::TopToolBarArea, fileToolbar);
        editToolbar = new QToolBar(qsamplerMainForm);
        editToolbar->setObjectName("editToolbar");
        editToolbar->setOrientation(Qt::Horizontal);
        qsamplerMainForm->addToolBar(Qt::ToolBarArea::TopToolBarArea, editToolbar);
        channelsToolbar = new QToolBar(qsamplerMainForm);
        channelsToolbar->setObjectName("channelsToolbar");
        channelsToolbar->setOrientation(Qt::Horizontal);
        qsamplerMainForm->addToolBar(Qt::ToolBarArea::TopToolBarArea, channelsToolbar);
        MenuBar = new QMenuBar(qsamplerMainForm);
        MenuBar->setObjectName("MenuBar");
        MenuBar->setGeometry(QRect(0, 0, 800, 31));
        editMenu = new QMenu(MenuBar);
        editMenu->setObjectName("editMenu");
        viewMenu = new QMenu(MenuBar);
        viewMenu->setObjectName("viewMenu");
        viewMidiDeviceStatusMenu = new QMenu(viewMenu);
        viewMidiDeviceStatusMenu->setObjectName("viewMidiDeviceStatusMenu");
        channelsMenu = new QMenu(MenuBar);
        channelsMenu->setObjectName("channelsMenu");
        helpMenu = new QMenu(MenuBar);
        helpMenu->setObjectName("helpMenu");
        fileMenu = new QMenu(MenuBar);
        fileMenu->setObjectName("fileMenu");
        fileOpenRecentMenu = new QMenu(fileMenu);
        fileOpenRecentMenu->setObjectName("fileOpenRecentMenu");
        qsamplerMainForm->setMenuBar(MenuBar);
        statusbar = new QStatusBar(qsamplerMainForm);
        statusbar->setObjectName("statusbar");
        qsamplerMainForm->setStatusBar(statusbar);

        fileToolbar->addAction(fileNewAction);
        fileToolbar->addAction(fileOpenAction);
        fileToolbar->addAction(fileSaveAction);
        fileToolbar->addSeparator();
        fileToolbar->addAction(fileResetAction);
        fileToolbar->addAction(fileRestartAction);
        fileToolbar->addSeparator();
        fileToolbar->addAction(viewInstrumentsAction);
        fileToolbar->addAction(viewDevicesAction);
        editToolbar->addAction(editAddChannelAction);
        editToolbar->addAction(editRemoveChannelAction);
        editToolbar->addSeparator();
        editToolbar->addAction(editSetupChannelAction);
        editToolbar->addAction(editEditChannelAction);
        editToolbar->addSeparator();
        editToolbar->addAction(editResetChannelAction);
        editToolbar->addAction(editResetAllChannelsAction);
        channelsToolbar->addAction(channelsArrangeAction);
        MenuBar->addAction(fileMenu->menuAction());
        MenuBar->addAction(editMenu->menuAction());
        MenuBar->addAction(viewMenu->menuAction());
        MenuBar->addAction(channelsMenu->menuAction());
        MenuBar->addSeparator();
        MenuBar->addAction(helpMenu->menuAction());
        editMenu->addAction(editAddChannelAction);
        editMenu->addAction(editRemoveChannelAction);
        editMenu->addSeparator();
        editMenu->addAction(editSetupChannelAction);
        editMenu->addAction(editEditChannelAction);
        editMenu->addSeparator();
        editMenu->addAction(editResetChannelAction);
        editMenu->addAction(editResetAllChannelsAction);
        viewMenu->addAction(viewMenubarAction);
        viewMenu->addAction(viewToolbarAction);
        viewMenu->addAction(viewStatusbarAction);
        viewMenu->addSeparator();
        viewMenu->addAction(viewMessagesAction);
        viewMenu->addAction(viewInstrumentsAction);
        viewMenu->addAction(viewDevicesAction);
        viewMenu->addSeparator();
        viewMenu->addAction(viewMidiDeviceStatusMenu->menuAction());
        viewMenu->addSeparator();
        viewMenu->addAction(viewOptionsAction);
        channelsMenu->addAction(channelsArrangeAction);
        channelsMenu->addAction(channelsAutoArrangeAction);
        helpMenu->addAction(helpAboutAction);
        helpMenu->addAction(helpAboutQtAction);
        fileMenu->addAction(fileNewAction);
        fileMenu->addSeparator();
        fileMenu->addAction(fileOpenAction);
        fileMenu->addAction(fileOpenRecentMenu->menuAction());
        fileMenu->addSeparator();
        fileMenu->addAction(fileSaveAction);
        fileMenu->addAction(fileSaveAsAction);
        fileMenu->addSeparator();
        fileMenu->addAction(fileResetAction);
        fileMenu->addAction(fileRestartAction);
        fileMenu->addSeparator();
        fileMenu->addAction(fileExitAction);

        retranslateUi(qsamplerMainForm);

        QMetaObject::connectSlotsByName(qsamplerMainForm);
    } // setupUi

    void retranslateUi(QMainWindow *qsamplerMainForm)
    {
        fileNewAction->setText(QCoreApplication::translate("qsamplerMainForm", "&New", nullptr));
        fileNewAction->setIconText(QCoreApplication::translate("qsamplerMainForm", "New", nullptr));
#if QT_CONFIG(tooltip)
        fileNewAction->setToolTip(QCoreApplication::translate("qsamplerMainForm", "New session", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        fileNewAction->setStatusTip(QCoreApplication::translate("qsamplerMainForm", "New sampler session", nullptr));
#endif // QT_CONFIG(statustip)
#if QT_CONFIG(shortcut)
        fileNewAction->setShortcut(QCoreApplication::translate("qsamplerMainForm", "Ctrl+N", nullptr));
#endif // QT_CONFIG(shortcut)
        fileOpenAction->setText(QCoreApplication::translate("qsamplerMainForm", "&Open...", nullptr));
        fileOpenAction->setIconText(QCoreApplication::translate("qsamplerMainForm", "Open", nullptr));
#if QT_CONFIG(tooltip)
        fileOpenAction->setToolTip(QCoreApplication::translate("qsamplerMainForm", "Open session", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        fileOpenAction->setStatusTip(QCoreApplication::translate("qsamplerMainForm", "Open sampler session", nullptr));
#endif // QT_CONFIG(statustip)
#if QT_CONFIG(shortcut)
        fileOpenAction->setShortcut(QCoreApplication::translate("qsamplerMainForm", "Ctrl+O", nullptr));
#endif // QT_CONFIG(shortcut)
        fileSaveAction->setText(QCoreApplication::translate("qsamplerMainForm", "&Save", nullptr));
        fileSaveAction->setIconText(QCoreApplication::translate("qsamplerMainForm", "Save", nullptr));
#if QT_CONFIG(tooltip)
        fileSaveAction->setToolTip(QCoreApplication::translate("qsamplerMainForm", "Save session", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        fileSaveAction->setStatusTip(QCoreApplication::translate("qsamplerMainForm", "Save sampler session", nullptr));
#endif // QT_CONFIG(statustip)
#if QT_CONFIG(shortcut)
        fileSaveAction->setShortcut(QCoreApplication::translate("qsamplerMainForm", "Ctrl+S", nullptr));
#endif // QT_CONFIG(shortcut)
        fileSaveAsAction->setText(QCoreApplication::translate("qsamplerMainForm", "Save &As...", nullptr));
        fileSaveAsAction->setIconText(QCoreApplication::translate("qsamplerMainForm", "Save As", nullptr));
#if QT_CONFIG(statustip)
        fileSaveAsAction->setStatusTip(QCoreApplication::translate("qsamplerMainForm", "Save current sampler session with another name", nullptr));
#endif // QT_CONFIG(statustip)
#if QT_CONFIG(shortcut)
        fileSaveAsAction->setShortcut(QString());
#endif // QT_CONFIG(shortcut)
        fileResetAction->setText(QCoreApplication::translate("qsamplerMainForm", "Rese&t", nullptr));
        fileResetAction->setIconText(QCoreApplication::translate("qsamplerMainForm", "Reset", nullptr));
#if QT_CONFIG(tooltip)
        fileResetAction->setToolTip(QCoreApplication::translate("qsamplerMainForm", "Reset instance", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        fileResetAction->setStatusTip(QCoreApplication::translate("qsamplerMainForm", "Reset sampler instance", nullptr));
#endif // QT_CONFIG(statustip)
#if QT_CONFIG(shortcut)
        fileResetAction->setShortcut(QCoreApplication::translate("qsamplerMainForm", "Ctrl+R", nullptr));
#endif // QT_CONFIG(shortcut)
        fileRestartAction->setText(QCoreApplication::translate("qsamplerMainForm", "&Restart", nullptr));
        fileRestartAction->setIconText(QCoreApplication::translate("qsamplerMainForm", "Restart", nullptr));
#if QT_CONFIG(tooltip)
        fileRestartAction->setToolTip(QCoreApplication::translate("qsamplerMainForm", "Restart instance", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        fileRestartAction->setStatusTip(QCoreApplication::translate("qsamplerMainForm", "Restart sampler instance", nullptr));
#endif // QT_CONFIG(statustip)
#if QT_CONFIG(shortcut)
        fileRestartAction->setShortcut(QCoreApplication::translate("qsamplerMainForm", "Ctrl+Shift+R", nullptr));
#endif // QT_CONFIG(shortcut)
        fileExitAction->setText(QCoreApplication::translate("qsamplerMainForm", "E&xit", nullptr));
        fileExitAction->setIconText(QCoreApplication::translate("qsamplerMainForm", "Exit", nullptr));
#if QT_CONFIG(statustip)
        fileExitAction->setStatusTip(QCoreApplication::translate("qsamplerMainForm", "Exit this application program", nullptr));
#endif // QT_CONFIG(statustip)
#if QT_CONFIG(shortcut)
        fileExitAction->setShortcut(QString());
#endif // QT_CONFIG(shortcut)
        editAddChannelAction->setText(QCoreApplication::translate("qsamplerMainForm", "&Add Channel", nullptr));
        editAddChannelAction->setIconText(QCoreApplication::translate("qsamplerMainForm", "Add", nullptr));
#if QT_CONFIG(tooltip)
        editAddChannelAction->setToolTip(QCoreApplication::translate("qsamplerMainForm", "Add channel", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        editAddChannelAction->setStatusTip(QCoreApplication::translate("qsamplerMainForm", "Add a new sampler channel", nullptr));
#endif // QT_CONFIG(statustip)
#if QT_CONFIG(shortcut)
        editAddChannelAction->setShortcut(QCoreApplication::translate("qsamplerMainForm", "Ctrl+A", nullptr));
#endif // QT_CONFIG(shortcut)
        editRemoveChannelAction->setText(QCoreApplication::translate("qsamplerMainForm", "&Remove Channel", nullptr));
        editRemoveChannelAction->setIconText(QCoreApplication::translate("qsamplerMainForm", "Remove", nullptr));
#if QT_CONFIG(tooltip)
        editRemoveChannelAction->setToolTip(QCoreApplication::translate("qsamplerMainForm", "Remove channel", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        editRemoveChannelAction->setStatusTip(QCoreApplication::translate("qsamplerMainForm", "Remove current sampler channel", nullptr));
#endif // QT_CONFIG(statustip)
#if QT_CONFIG(shortcut)
        editRemoveChannelAction->setShortcut(QCoreApplication::translate("qsamplerMainForm", "Ctrl+X", nullptr));
#endif // QT_CONFIG(shortcut)
        editResetChannelAction->setText(QCoreApplication::translate("qsamplerMainForm", "Re&set Channel", nullptr));
        editResetChannelAction->setIconText(QCoreApplication::translate("qsamplerMainForm", "Reset", nullptr));
#if QT_CONFIG(tooltip)
        editResetChannelAction->setToolTip(QCoreApplication::translate("qsamplerMainForm", "Reset channel", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        editResetChannelAction->setStatusTip(QCoreApplication::translate("qsamplerMainForm", "Reset current sampler channel", nullptr));
#endif // QT_CONFIG(statustip)
#if QT_CONFIG(shortcut)
        editResetChannelAction->setShortcut(QString());
#endif // QT_CONFIG(shortcut)
        editResetAllChannelsAction->setText(QCoreApplication::translate("qsamplerMainForm", "R&eset All Channels", nullptr));
        editResetAllChannelsAction->setIconText(QCoreApplication::translate("qsamplerMainForm", "Reset All", nullptr));
#if QT_CONFIG(tooltip)
        editResetAllChannelsAction->setToolTip(QCoreApplication::translate("qsamplerMainForm", "Reset all channels", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        editResetAllChannelsAction->setStatusTip(QCoreApplication::translate("qsamplerMainForm", "Reset all sampler channels", nullptr));
#endif // QT_CONFIG(statustip)
#if QT_CONFIG(shortcut)
        editResetAllChannelsAction->setShortcut(QString());
#endif // QT_CONFIG(shortcut)
        editSetupChannelAction->setText(QCoreApplication::translate("qsamplerMainForm", "&Setup Channel...", nullptr));
        editSetupChannelAction->setIconText(QCoreApplication::translate("qsamplerMainForm", "Setup", nullptr));
#if QT_CONFIG(tooltip)
        editSetupChannelAction->setToolTip(QCoreApplication::translate("qsamplerMainForm", "Setup channel", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        editSetupChannelAction->setStatusTip(QCoreApplication::translate("qsamplerMainForm", "Setup current sampler channel", nullptr));
#endif // QT_CONFIG(statustip)
#if QT_CONFIG(shortcut)
        editSetupChannelAction->setShortcut(QCoreApplication::translate("qsamplerMainForm", "F2", nullptr));
#endif // QT_CONFIG(shortcut)
        editEditChannelAction->setText(QCoreApplication::translate("qsamplerMainForm", "Ed&it Channel...", nullptr));
        editEditChannelAction->setIconText(QCoreApplication::translate("qsamplerMainForm", "Edit", nullptr));
#if QT_CONFIG(tooltip)
        editEditChannelAction->setToolTip(QCoreApplication::translate("qsamplerMainForm", "Edit channel", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        editEditChannelAction->setStatusTip(QCoreApplication::translate("qsamplerMainForm", "Edit current sampler channel", nullptr));
#endif // QT_CONFIG(statustip)
#if QT_CONFIG(shortcut)
        editEditChannelAction->setShortcut(QCoreApplication::translate("qsamplerMainForm", "F9", nullptr));
#endif // QT_CONFIG(shortcut)
        viewMenubarAction->setText(QCoreApplication::translate("qsamplerMainForm", "&Menubar", nullptr));
        viewMenubarAction->setIconText(QCoreApplication::translate("qsamplerMainForm", "Menubar", nullptr));
#if QT_CONFIG(tooltip)
        viewMenubarAction->setToolTip(QCoreApplication::translate("qsamplerMainForm", "Show/hide menubar", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        viewMenubarAction->setStatusTip(QCoreApplication::translate("qsamplerMainForm", "Show/hide the main program window menubar", nullptr));
#endif // QT_CONFIG(statustip)
#if QT_CONFIG(shortcut)
        viewMenubarAction->setShortcut(QCoreApplication::translate("qsamplerMainForm", "Ctrl+M", nullptr));
#endif // QT_CONFIG(shortcut)
        viewToolbarAction->setText(QCoreApplication::translate("qsamplerMainForm", "&Toolbar", nullptr));
        viewToolbarAction->setIconText(QCoreApplication::translate("qsamplerMainForm", "Toolbar", nullptr));
#if QT_CONFIG(tooltip)
        viewToolbarAction->setToolTip(QCoreApplication::translate("qsamplerMainForm", "Show/hide toolbar", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        viewToolbarAction->setStatusTip(QCoreApplication::translate("qsamplerMainForm", "Show/hide main program window toolbars", nullptr));
#endif // QT_CONFIG(statustip)
#if QT_CONFIG(shortcut)
        viewToolbarAction->setShortcut(QCoreApplication::translate("qsamplerMainForm", "Ctrl+T", nullptr));
#endif // QT_CONFIG(shortcut)
        viewStatusbarAction->setText(QCoreApplication::translate("qsamplerMainForm", "&Statusbar", nullptr));
        viewStatusbarAction->setIconText(QCoreApplication::translate("qsamplerMainForm", "Statusbar", nullptr));
#if QT_CONFIG(tooltip)
        viewStatusbarAction->setToolTip(QCoreApplication::translate("qsamplerMainForm", "Show/hide statusbar", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        viewStatusbarAction->setStatusTip(QCoreApplication::translate("qsamplerMainForm", "Show/hide the main program window statusbar", nullptr));
#endif // QT_CONFIG(statustip)
#if QT_CONFIG(shortcut)
        viewStatusbarAction->setShortcut(QString());
#endif // QT_CONFIG(shortcut)
        viewMessagesAction->setText(QCoreApplication::translate("qsamplerMainForm", "M&essages", nullptr));
        viewMessagesAction->setIconText(QCoreApplication::translate("qsamplerMainForm", "Messages", nullptr));
#if QT_CONFIG(tooltip)
        viewMessagesAction->setToolTip(QCoreApplication::translate("qsamplerMainForm", "Show/hide messages", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        viewMessagesAction->setStatusTip(QCoreApplication::translate("qsamplerMainForm", "Show/hide the messages window", nullptr));
#endif // QT_CONFIG(statustip)
#if QT_CONFIG(shortcut)
        viewMessagesAction->setShortcut(QString());
#endif // QT_CONFIG(shortcut)
        viewInstrumentsAction->setText(QCoreApplication::translate("qsamplerMainForm", "&Instruments", nullptr));
        viewInstrumentsAction->setIconText(QCoreApplication::translate("qsamplerMainForm", "Instruments", nullptr));
#if QT_CONFIG(tooltip)
        viewInstrumentsAction->setToolTip(QCoreApplication::translate("qsamplerMainForm", "MIDI instruments configuration", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        viewInstrumentsAction->setStatusTip(QCoreApplication::translate("qsamplerMainForm", "Show/hide the MIDI instruments configuration window", nullptr));
#endif // QT_CONFIG(statustip)
#if QT_CONFIG(shortcut)
        viewInstrumentsAction->setShortcut(QCoreApplication::translate("qsamplerMainForm", "F10", nullptr));
#endif // QT_CONFIG(shortcut)
        viewDevicesAction->setText(QCoreApplication::translate("qsamplerMainForm", "&Devices", nullptr));
        viewDevicesAction->setIconText(QCoreApplication::translate("qsamplerMainForm", "Devices", nullptr));
#if QT_CONFIG(tooltip)
        viewDevicesAction->setToolTip(QCoreApplication::translate("qsamplerMainForm", "Device configuration", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        viewDevicesAction->setStatusTip(QCoreApplication::translate("qsamplerMainForm", "Show/hide the device configuration window", nullptr));
#endif // QT_CONFIG(statustip)
#if QT_CONFIG(shortcut)
        viewDevicesAction->setShortcut(QCoreApplication::translate("qsamplerMainForm", "F11", nullptr));
#endif // QT_CONFIG(shortcut)
        viewOptionsAction->setText(QCoreApplication::translate("qsamplerMainForm", "&Options...", nullptr));
        viewOptionsAction->setIconText(QCoreApplication::translate("qsamplerMainForm", "Options", nullptr));
#if QT_CONFIG(tooltip)
        viewOptionsAction->setToolTip(QCoreApplication::translate("qsamplerMainForm", "General options", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        viewOptionsAction->setStatusTip(QCoreApplication::translate("qsamplerMainForm", "Change general application program options", nullptr));
#endif // QT_CONFIG(statustip)
#if QT_CONFIG(shortcut)
        viewOptionsAction->setShortcut(QCoreApplication::translate("qsamplerMainForm", "F12", nullptr));
#endif // QT_CONFIG(shortcut)
        channelsArrangeAction->setText(QCoreApplication::translate("qsamplerMainForm", "&Arrange", nullptr));
        channelsArrangeAction->setIconText(QCoreApplication::translate("qsamplerMainForm", "Arrange", nullptr));
#if QT_CONFIG(tooltip)
        channelsArrangeAction->setToolTip(QCoreApplication::translate("qsamplerMainForm", "Arrange channels", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        channelsArrangeAction->setStatusTip(QCoreApplication::translate("qsamplerMainForm", "Line up all channel strips", nullptr));
#endif // QT_CONFIG(statustip)
#if QT_CONFIG(shortcut)
        channelsArrangeAction->setShortcut(QCoreApplication::translate("qsamplerMainForm", "F5", nullptr));
#endif // QT_CONFIG(shortcut)
        channelsAutoArrangeAction->setText(QCoreApplication::translate("qsamplerMainForm", "A&uto Arrange", nullptr));
        channelsAutoArrangeAction->setIconText(QCoreApplication::translate("qsamplerMainForm", "Auto Arrange", nullptr));
#if QT_CONFIG(tooltip)
        channelsAutoArrangeAction->setToolTip(QCoreApplication::translate("qsamplerMainForm", "Auto-arrange channels", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        channelsAutoArrangeAction->setStatusTip(QCoreApplication::translate("qsamplerMainForm", "Auto-arrange channel strips", nullptr));
#endif // QT_CONFIG(statustip)
#if QT_CONFIG(shortcut)
        channelsAutoArrangeAction->setShortcut(QString());
#endif // QT_CONFIG(shortcut)
        helpAboutAction->setText(QCoreApplication::translate("qsamplerMainForm", "&About...", nullptr));
        helpAboutAction->setIconText(QCoreApplication::translate("qsamplerMainForm", "About", nullptr));
#if QT_CONFIG(statustip)
        helpAboutAction->setStatusTip(QCoreApplication::translate("qsamplerMainForm", "Show information about this application program", nullptr));
#endif // QT_CONFIG(statustip)
#if QT_CONFIG(shortcut)
        helpAboutAction->setShortcut(QString());
#endif // QT_CONFIG(shortcut)
        helpAboutQtAction->setText(QCoreApplication::translate("qsamplerMainForm", "About &Qt...", nullptr));
        helpAboutQtAction->setIconText(QCoreApplication::translate("qsamplerMainForm", "About Qt", nullptr));
#if QT_CONFIG(statustip)
        helpAboutQtAction->setStatusTip(QCoreApplication::translate("qsamplerMainForm", "Show information about the Qt toolkit", nullptr));
#endif // QT_CONFIG(statustip)
#if QT_CONFIG(shortcut)
        helpAboutQtAction->setShortcut(QString());
#endif // QT_CONFIG(shortcut)
        editMenu->setTitle(QCoreApplication::translate("qsamplerMainForm", "&Edit", nullptr));
        viewMenu->setTitle(QCoreApplication::translate("qsamplerMainForm", "&View", nullptr));
        viewMidiDeviceStatusMenu->setTitle(QCoreApplication::translate("qsamplerMainForm", "MIDI Device Status", nullptr));
        channelsMenu->setTitle(QCoreApplication::translate("qsamplerMainForm", "&Channels", nullptr));
        helpMenu->setTitle(QCoreApplication::translate("qsamplerMainForm", "&Help", nullptr));
        fileMenu->setTitle(QCoreApplication::translate("qsamplerMainForm", "&File", nullptr));
        fileOpenRecentMenu->setTitle(QCoreApplication::translate("qsamplerMainForm", "Open &Recent", nullptr));
        (void)qsamplerMainForm;
    } // retranslateUi

};

namespace Ui {
    class qsamplerMainForm: public Ui_qsamplerMainForm {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_QSAMPLERMAINFORM_H
