/*
qsampler - A LinuxSampler Qt GUI Interface.

   Copyright (C) 2005-2020, rncbc aka Rui Nuno Capela. All rights reserved.
   Copyright (C) 2007, Christian Schoenebeck

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


*/

/********************************************************************************
** Form generated from reading UI file 'qsamplerChannelForm.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_QSAMPLERCHANNELFORM_H
#define UI_QSAMPLERCHANNELFORM_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QTableView>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_qsamplerChannelForm
{
public:
    QVBoxLayout *vboxLayout;
    QGridLayout *gridLayout;
    QLabel *EngineNameTextLabel;
    QComboBox *EngineNameComboBox;
    QSpacerItem *spacerItem;
    QLabel *InstrumentFileTextLabel;
    QComboBox *InstrumentFileComboBox;
    QToolButton *InstrumentFileToolButton;
    QLabel *InstrumentNrTextLabel;
    QComboBox *InstrumentNrComboBox;
    QGroupBox *MidiGroupBox;
    QGridLayout *gridLayout1;
    QLabel *MidiDriverTextLabel;
    QComboBox *MidiDriverComboBox;
    QSpacerItem *spacerItem1;
    QLabel *MidiPortTextLabel;
    QSpinBox *MidiPortSpinBox;
    QLabel *MidiChannelTextLabel;
    QComboBox *MidiChannelComboBox;
    QSpacerItem *spacerItem2;
    QLabel *MidiDeviceTextLabel;
    QComboBox *MidiDeviceComboBox;
    QToolButton *MidiDeviceToolButton;
    QLabel *MidiMapTextLabel;
    QComboBox *MidiMapComboBox;
    QSpacerItem *spacerItem3;
    QGroupBox *AudioGroupBox;
    QGridLayout *gridLayout2;
    QLabel *AudioDriverTextLabel;
    QComboBox *AudioDriverComboBox;
    QSpacerItem *spacerItem4;
    QLabel *AudioDeviceTextLabel;
    QComboBox *AudioDeviceComboBox;
    QToolButton *AudioDeviceToolButton;
    QTableView *AudioRoutingTable;
    QDialogButtonBox *DialogButtonBox;

    void setupUi(QDialog *qsamplerChannelForm)
    {
        if (qsamplerChannelForm->objectName().isEmpty())
            qsamplerChannelForm->setObjectName("qsamplerChannelForm");
        qsamplerChannelForm->resize(480, 460);
        qsamplerChannelForm->setFocusPolicy(Qt::StrongFocus);
        const QIcon icon = QIcon(QString::fromUtf8(":/images/qsamplerChannel.png"));
        qsamplerChannelForm->setWindowIcon(icon);
        vboxLayout = new QVBoxLayout(qsamplerChannelForm);
        vboxLayout->setSpacing(4);
        vboxLayout->setContentsMargins(8, 8, 8, 8);
        vboxLayout->setObjectName("vboxLayout");
        gridLayout = new QGridLayout();
        gridLayout->setSpacing(4);
        gridLayout->setContentsMargins(8, 8, 8, 8);
        gridLayout->setObjectName("gridLayout");
        EngineNameTextLabel = new QLabel(qsamplerChannelForm);
        EngineNameTextLabel->setObjectName("EngineNameTextLabel");
        EngineNameTextLabel->setMinimumSize(QSize(62, 0));
        EngineNameTextLabel->setMaximumSize(QSize(62, 16777215));
        EngineNameTextLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        EngineNameTextLabel->setWordWrap(false);

        gridLayout->addWidget(EngineNameTextLabel, 0, 0, 1, 1);

        EngineNameComboBox = new QComboBox(qsamplerChannelForm);
        EngineNameComboBox->setObjectName("EngineNameComboBox");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(EngineNameComboBox->sizePolicy().hasHeightForWidth());
        EngineNameComboBox->setSizePolicy(sizePolicy);

        gridLayout->addWidget(EngineNameComboBox, 0, 1, 1, 1);

        spacerItem = new QSpacerItem(20, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout->addItem(spacerItem, 0, 2, 1, 1);

        InstrumentFileTextLabel = new QLabel(qsamplerChannelForm);
        InstrumentFileTextLabel->setObjectName("InstrumentFileTextLabel");
        InstrumentFileTextLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        InstrumentFileTextLabel->setWordWrap(false);

        gridLayout->addWidget(InstrumentFileTextLabel, 1, 0, 1, 1);

        InstrumentFileComboBox = new QComboBox(qsamplerChannelForm);
        InstrumentFileComboBox->setObjectName("InstrumentFileComboBox");
        sizePolicy.setHeightForWidth(InstrumentFileComboBox->sizePolicy().hasHeightForWidth());
        InstrumentFileComboBox->setSizePolicy(sizePolicy);
        InstrumentFileComboBox->setMinimumSize(QSize(320, 0));
        InstrumentFileComboBox->setEditable(true);

        gridLayout->addWidget(InstrumentFileComboBox, 1, 1, 1, 2);

        InstrumentFileToolButton = new QToolButton(qsamplerChannelForm);
        InstrumentFileToolButton->setObjectName("InstrumentFileToolButton");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(InstrumentFileToolButton->sizePolicy().hasHeightForWidth());
        InstrumentFileToolButton->setSizePolicy(sizePolicy1);
        InstrumentFileToolButton->setMinimumSize(QSize(24, 24));
        InstrumentFileToolButton->setMaximumSize(QSize(26, 26));
        InstrumentFileToolButton->setFocusPolicy(Qt::TabFocus);
        const QIcon icon1 = QIcon(QString::fromUtf8(":/images/fileOpen.png"));
        InstrumentFileToolButton->setIcon(icon1);

        gridLayout->addWidget(InstrumentFileToolButton, 1, 3, 1, 1);

        InstrumentNrTextLabel = new QLabel(qsamplerChannelForm);
        InstrumentNrTextLabel->setObjectName("InstrumentNrTextLabel");
        InstrumentNrTextLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        InstrumentNrTextLabel->setWordWrap(false);

        gridLayout->addWidget(InstrumentNrTextLabel, 2, 0, 1, 1);

        InstrumentNrComboBox = new QComboBox(qsamplerChannelForm);
        InstrumentNrComboBox->setObjectName("InstrumentNrComboBox");
        sizePolicy.setHeightForWidth(InstrumentNrComboBox->sizePolicy().hasHeightForWidth());
        InstrumentNrComboBox->setSizePolicy(sizePolicy);
        InstrumentNrComboBox->setMinimumSize(QSize(320, 0));

        gridLayout->addWidget(InstrumentNrComboBox, 2, 1, 1, 2);


        vboxLayout->addLayout(gridLayout);

        MidiGroupBox = new QGroupBox(qsamplerChannelForm);
        MidiGroupBox->setObjectName("MidiGroupBox");
        gridLayout1 = new QGridLayout(MidiGroupBox);
        gridLayout1->setSpacing(4);
        gridLayout1->setContentsMargins(8, 8, 8, 8);
        gridLayout1->setObjectName("gridLayout1");
        MidiDriverTextLabel = new QLabel(MidiGroupBox);
        MidiDriverTextLabel->setObjectName("MidiDriverTextLabel");
        MidiDriverTextLabel->setMinimumSize(QSize(62, 0));
        MidiDriverTextLabel->setMaximumSize(QSize(62, 16777215));
        MidiDriverTextLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        MidiDriverTextLabel->setWordWrap(false);

        gridLayout1->addWidget(MidiDriverTextLabel, 0, 0, 1, 1);

        MidiDriverComboBox = new QComboBox(MidiGroupBox);
        MidiDriverComboBox->setObjectName("MidiDriverComboBox");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Fixed);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(MidiDriverComboBox->sizePolicy().hasHeightForWidth());
        MidiDriverComboBox->setSizePolicy(sizePolicy2);
        MidiDriverComboBox->setMinimumSize(QSize(92, 0));
        MidiDriverComboBox->setMaximumSize(QSize(92, 16777215));

        gridLayout1->addWidget(MidiDriverComboBox, 0, 1, 1, 1);

        spacerItem1 = new QSpacerItem(20, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout1->addItem(spacerItem1, 0, 2, 1, 1);

        MidiPortTextLabel = new QLabel(MidiGroupBox);
        MidiPortTextLabel->setObjectName("MidiPortTextLabel");
        MidiPortTextLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        MidiPortTextLabel->setWordWrap(false);

        gridLayout1->addWidget(MidiPortTextLabel, 0, 3, 1, 1);

        MidiPortSpinBox = new QSpinBox(MidiGroupBox);
        MidiPortSpinBox->setObjectName("MidiPortSpinBox");

        gridLayout1->addWidget(MidiPortSpinBox, 0, 4, 1, 1);

        MidiChannelTextLabel = new QLabel(MidiGroupBox);
        MidiChannelTextLabel->setObjectName("MidiChannelTextLabel");
        MidiChannelTextLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        MidiChannelTextLabel->setWordWrap(false);

        gridLayout1->addWidget(MidiChannelTextLabel, 0, 5, 1, 1);

        MidiChannelComboBox = new QComboBox(MidiGroupBox);
        MidiChannelComboBox->addItem(QString());
        MidiChannelComboBox->addItem(QString());
        MidiChannelComboBox->addItem(QString());
        MidiChannelComboBox->addItem(QString());
        MidiChannelComboBox->addItem(QString());
        MidiChannelComboBox->addItem(QString());
        MidiChannelComboBox->addItem(QString());
        MidiChannelComboBox->addItem(QString());
        MidiChannelComboBox->addItem(QString());
        MidiChannelComboBox->addItem(QString());
        MidiChannelComboBox->addItem(QString());
        MidiChannelComboBox->addItem(QString());
        MidiChannelComboBox->addItem(QString());
        MidiChannelComboBox->addItem(QString());
        MidiChannelComboBox->addItem(QString());
        MidiChannelComboBox->addItem(QString());
        MidiChannelComboBox->addItem(QString());
        MidiChannelComboBox->setObjectName("MidiChannelComboBox");

        gridLayout1->addWidget(MidiChannelComboBox, 0, 6, 1, 1);

        spacerItem2 = new QSpacerItem(20, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout1->addItem(spacerItem2, 0, 7, 1, 1);

        MidiDeviceTextLabel = new QLabel(MidiGroupBox);
        MidiDeviceTextLabel->setObjectName("MidiDeviceTextLabel");
        MidiDeviceTextLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        MidiDeviceTextLabel->setWordWrap(false);

        gridLayout1->addWidget(MidiDeviceTextLabel, 1, 0, 1, 1);

        MidiDeviceComboBox = new QComboBox(MidiGroupBox);
        MidiDeviceComboBox->setObjectName("MidiDeviceComboBox");
        sizePolicy.setHeightForWidth(MidiDeviceComboBox->sizePolicy().hasHeightForWidth());
        MidiDeviceComboBox->setSizePolicy(sizePolicy);

        gridLayout1->addWidget(MidiDeviceComboBox, 1, 1, 1, 6);

        MidiDeviceToolButton = new QToolButton(MidiGroupBox);
        MidiDeviceToolButton->setObjectName("MidiDeviceToolButton");
        sizePolicy1.setHeightForWidth(MidiDeviceToolButton->sizePolicy().hasHeightForWidth());
        MidiDeviceToolButton->setSizePolicy(sizePolicy1);
        MidiDeviceToolButton->setMinimumSize(QSize(24, 24));
        MidiDeviceToolButton->setMaximumSize(QSize(26, 26));
        MidiDeviceToolButton->setFocusPolicy(Qt::TabFocus);
        const QIcon icon2 = QIcon(QString::fromUtf8(":/images/midi1.png"));
        MidiDeviceToolButton->setIcon(icon2);

        gridLayout1->addWidget(MidiDeviceToolButton, 1, 7, 1, 1);

        MidiMapTextLabel = new QLabel(MidiGroupBox);
        MidiMapTextLabel->setObjectName("MidiMapTextLabel");
        MidiMapTextLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        MidiMapTextLabel->setWordWrap(false);

        gridLayout1->addWidget(MidiMapTextLabel, 2, 0, 1, 1);

        MidiMapComboBox = new QComboBox(MidiGroupBox);
        MidiMapComboBox->setObjectName("MidiMapComboBox");
        sizePolicy.setHeightForWidth(MidiMapComboBox->sizePolicy().hasHeightForWidth());
        MidiMapComboBox->setSizePolicy(sizePolicy);

        gridLayout1->addWidget(MidiMapComboBox, 2, 1, 1, 2);

        spacerItem3 = new QSpacerItem(20, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout1->addItem(spacerItem3, 2, 3, 1, 5);


        vboxLayout->addWidget(MidiGroupBox);

        AudioGroupBox = new QGroupBox(qsamplerChannelForm);
        AudioGroupBox->setObjectName("AudioGroupBox");
        gridLayout2 = new QGridLayout(AudioGroupBox);
        gridLayout2->setSpacing(4);
        gridLayout2->setContentsMargins(8, 8, 8, 8);
        gridLayout2->setObjectName("gridLayout2");
        AudioDriverTextLabel = new QLabel(AudioGroupBox);
        AudioDriverTextLabel->setObjectName("AudioDriverTextLabel");
        AudioDriverTextLabel->setMinimumSize(QSize(62, 0));
        AudioDriverTextLabel->setMaximumSize(QSize(62, 16777215));
        AudioDriverTextLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        AudioDriverTextLabel->setWordWrap(false);

        gridLayout2->addWidget(AudioDriverTextLabel, 0, 0, 1, 1);

        AudioDriverComboBox = new QComboBox(AudioGroupBox);
        AudioDriverComboBox->setObjectName("AudioDriverComboBox");
        sizePolicy2.setHeightForWidth(AudioDriverComboBox->sizePolicy().hasHeightForWidth());
        AudioDriverComboBox->setSizePolicy(sizePolicy2);
        AudioDriverComboBox->setMinimumSize(QSize(92, 0));
        AudioDriverComboBox->setMaximumSize(QSize(92, 16777215));

        gridLayout2->addWidget(AudioDriverComboBox, 0, 1, 1, 1);

        spacerItem4 = new QSpacerItem(20, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout2->addItem(spacerItem4, 0, 2, 1, 2);

        AudioDeviceTextLabel = new QLabel(AudioGroupBox);
        AudioDeviceTextLabel->setObjectName("AudioDeviceTextLabel");
        AudioDeviceTextLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        AudioDeviceTextLabel->setWordWrap(false);

        gridLayout2->addWidget(AudioDeviceTextLabel, 1, 0, 1, 1);

        AudioDeviceComboBox = new QComboBox(AudioGroupBox);
        AudioDeviceComboBox->setObjectName("AudioDeviceComboBox");
        sizePolicy.setHeightForWidth(AudioDeviceComboBox->sizePolicy().hasHeightForWidth());
        AudioDeviceComboBox->setSizePolicy(sizePolicy);

        gridLayout2->addWidget(AudioDeviceComboBox, 1, 1, 1, 2);

        AudioDeviceToolButton = new QToolButton(AudioGroupBox);
        AudioDeviceToolButton->setObjectName("AudioDeviceToolButton");
        sizePolicy1.setHeightForWidth(AudioDeviceToolButton->sizePolicy().hasHeightForWidth());
        AudioDeviceToolButton->setSizePolicy(sizePolicy1);
        AudioDeviceToolButton->setMinimumSize(QSize(24, 24));
        AudioDeviceToolButton->setMaximumSize(QSize(26, 26));
        AudioDeviceToolButton->setFocusPolicy(Qt::TabFocus);
        const QIcon icon3 = QIcon(QString::fromUtf8(":/images/audio1.png"));
        AudioDeviceToolButton->setIcon(icon3);

        gridLayout2->addWidget(AudioDeviceToolButton, 1, 3, 1, 1);

        AudioRoutingTable = new QTableView(AudioGroupBox);
        AudioRoutingTable->setObjectName("AudioRoutingTable");
        QSizePolicy sizePolicy3(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy3.setHorizontalStretch(0);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(AudioRoutingTable->sizePolicy().hasHeightForWidth());
        AudioRoutingTable->setSizePolicy(sizePolicy3);
        AudioRoutingTable->setMinimumSize(QSize(320, 80));

        gridLayout2->addWidget(AudioRoutingTable, 2, 0, 1, 4);


        vboxLayout->addWidget(AudioGroupBox);

        DialogButtonBox = new QDialogButtonBox(qsamplerChannelForm);
        DialogButtonBox->setObjectName("DialogButtonBox");
        DialogButtonBox->setOrientation(Qt::Horizontal);
        DialogButtonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        vboxLayout->addWidget(DialogButtonBox);

#if QT_CONFIG(shortcut)
        EngineNameTextLabel->setBuddy(EngineNameComboBox);
        InstrumentFileTextLabel->setBuddy(InstrumentFileComboBox);
        InstrumentNrTextLabel->setBuddy(InstrumentNrComboBox);
        MidiDriverTextLabel->setBuddy(MidiDriverComboBox);
        MidiPortTextLabel->setBuddy(MidiPortSpinBox);
        MidiChannelTextLabel->setBuddy(MidiChannelComboBox);
        MidiDeviceTextLabel->setBuddy(MidiDeviceComboBox);
        MidiMapTextLabel->setBuddy(MidiMapComboBox);
        AudioDriverTextLabel->setBuddy(AudioDriverComboBox);
        AudioDeviceTextLabel->setBuddy(AudioDeviceComboBox);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(EngineNameComboBox, InstrumentFileComboBox);
        QWidget::setTabOrder(InstrumentFileComboBox, InstrumentFileToolButton);
        QWidget::setTabOrder(InstrumentFileToolButton, InstrumentNrComboBox);
        QWidget::setTabOrder(InstrumentNrComboBox, DialogButtonBox);

        retranslateUi(qsamplerChannelForm);

        QMetaObject::connectSlotsByName(qsamplerChannelForm);
    } // setupUi

    void retranslateUi(QDialog *qsamplerChannelForm)
    {
        qsamplerChannelForm->setWindowTitle(QCoreApplication::translate("qsamplerChannelForm", "Channel", nullptr));
        EngineNameTextLabel->setText(QCoreApplication::translate("qsamplerChannelForm", "&Engine:", nullptr));
#if QT_CONFIG(tooltip)
        EngineNameComboBox->setToolTip(QCoreApplication::translate("qsamplerChannelForm", "Engine name", nullptr));
#endif // QT_CONFIG(tooltip)
        InstrumentFileTextLabel->setText(QCoreApplication::translate("qsamplerChannelForm", "Filena&me:", nullptr));
#if QT_CONFIG(tooltip)
        InstrumentFileComboBox->setToolTip(QCoreApplication::translate("qsamplerChannelForm", "Instrument filename", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        InstrumentFileToolButton->setToolTip(QCoreApplication::translate("qsamplerChannelForm", "Browse for instrument filename", nullptr));
#endif // QT_CONFIG(tooltip)
        InstrumentFileToolButton->setText(QString());
        InstrumentNrTextLabel->setText(QCoreApplication::translate("qsamplerChannelForm", "&Instrument:", nullptr));
#if QT_CONFIG(tooltip)
        InstrumentNrComboBox->setToolTip(QCoreApplication::translate("qsamplerChannelForm", "Instrument name", nullptr));
#endif // QT_CONFIG(tooltip)
        MidiGroupBox->setTitle(QCoreApplication::translate("qsamplerChannelForm", "MIDI / Input", nullptr));
        MidiDriverTextLabel->setText(QCoreApplication::translate("qsamplerChannelForm", "T&ype:", nullptr));
#if QT_CONFIG(tooltip)
        MidiDriverComboBox->setToolTip(QCoreApplication::translate("qsamplerChannelForm", "MIDI input driver type", nullptr));
#endif // QT_CONFIG(tooltip)
        MidiPortTextLabel->setText(QCoreApplication::translate("qsamplerChannelForm", "&Port:", nullptr));
#if QT_CONFIG(tooltip)
        MidiPortSpinBox->setToolTip(QCoreApplication::translate("qsamplerChannelForm", "MIDI input port number", nullptr));
#endif // QT_CONFIG(tooltip)
        MidiChannelTextLabel->setText(QCoreApplication::translate("qsamplerChannelForm", "&Channel:", nullptr));
        MidiChannelComboBox->setItemText(0, QCoreApplication::translate("qsamplerChannelForm", "1", nullptr));
        MidiChannelComboBox->setItemText(1, QCoreApplication::translate("qsamplerChannelForm", "2", nullptr));
        MidiChannelComboBox->setItemText(2, QCoreApplication::translate("qsamplerChannelForm", "3", nullptr));
        MidiChannelComboBox->setItemText(3, QCoreApplication::translate("qsamplerChannelForm", "4", nullptr));
        MidiChannelComboBox->setItemText(4, QCoreApplication::translate("qsamplerChannelForm", "5", nullptr));
        MidiChannelComboBox->setItemText(5, QCoreApplication::translate("qsamplerChannelForm", "6", nullptr));
        MidiChannelComboBox->setItemText(6, QCoreApplication::translate("qsamplerChannelForm", "7", nullptr));
        MidiChannelComboBox->setItemText(7, QCoreApplication::translate("qsamplerChannelForm", "8", nullptr));
        MidiChannelComboBox->setItemText(8, QCoreApplication::translate("qsamplerChannelForm", "9", nullptr));
        MidiChannelComboBox->setItemText(9, QCoreApplication::translate("qsamplerChannelForm", "10", nullptr));
        MidiChannelComboBox->setItemText(10, QCoreApplication::translate("qsamplerChannelForm", "11", nullptr));
        MidiChannelComboBox->setItemText(11, QCoreApplication::translate("qsamplerChannelForm", "12", nullptr));
        MidiChannelComboBox->setItemText(12, QCoreApplication::translate("qsamplerChannelForm", "13", nullptr));
        MidiChannelComboBox->setItemText(13, QCoreApplication::translate("qsamplerChannelForm", "14", nullptr));
        MidiChannelComboBox->setItemText(14, QCoreApplication::translate("qsamplerChannelForm", "15", nullptr));
        MidiChannelComboBox->setItemText(15, QCoreApplication::translate("qsamplerChannelForm", "16", nullptr));
        MidiChannelComboBox->setItemText(16, QCoreApplication::translate("qsamplerChannelForm", "All", nullptr));

#if QT_CONFIG(tooltip)
        MidiChannelComboBox->setToolTip(QCoreApplication::translate("qsamplerChannelForm", "MIDI input channel", nullptr));
#endif // QT_CONFIG(tooltip)
        MidiDeviceTextLabel->setText(QCoreApplication::translate("qsamplerChannelForm", "De&vice:", nullptr));
#if QT_CONFIG(tooltip)
        MidiDeviceComboBox->setToolTip(QCoreApplication::translate("qsamplerChannelForm", "MIDI input device", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        MidiDeviceToolButton->setToolTip(QCoreApplication::translate("qsamplerChannelForm", "MIDI input device setup", nullptr));
#endif // QT_CONFIG(tooltip)
        MidiDeviceToolButton->setText(QString());
        MidiMapTextLabel->setText(QCoreApplication::translate("qsamplerChannelForm", "&Map:", nullptr));
#if QT_CONFIG(tooltip)
        MidiMapComboBox->setToolTip(QCoreApplication::translate("qsamplerChannelForm", "Instrument map", nullptr));
#endif // QT_CONFIG(tooltip)
        AudioGroupBox->setTitle(QCoreApplication::translate("qsamplerChannelForm", "Audio / Output", nullptr));
        AudioDriverTextLabel->setText(QCoreApplication::translate("qsamplerChannelForm", "T&ype:", nullptr));
#if QT_CONFIG(tooltip)
        AudioDriverComboBox->setToolTip(QCoreApplication::translate("qsamplerChannelForm", "Audio output driver type", nullptr));
#endif // QT_CONFIG(tooltip)
        AudioDeviceTextLabel->setText(QCoreApplication::translate("qsamplerChannelForm", "De&vice:", nullptr));
#if QT_CONFIG(tooltip)
        AudioDeviceComboBox->setToolTip(QCoreApplication::translate("qsamplerChannelForm", "Audio output device", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        AudioDeviceToolButton->setToolTip(QCoreApplication::translate("qsamplerChannelForm", "Audio output device setup", nullptr));
#endif // QT_CONFIG(tooltip)
        AudioDeviceToolButton->setText(QString());
#if QT_CONFIG(tooltip)
        AudioRoutingTable->setToolTip(QCoreApplication::translate("qsamplerChannelForm", "Audio routing table", nullptr));
#endif // QT_CONFIG(tooltip)
    } // retranslateUi

};

namespace Ui {
    class qsamplerChannelForm: public Ui_qsamplerChannelForm {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_QSAMPLERCHANNELFORM_H
