/*
qsampler - A LinuxSampler Qt GUI Interface.

   Copyright (C) 2005-2022, rncbc aka Rui Nuno Capela. All rights reserved.
   Copyright (C) 2007, Christian Schoenebeck

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

 
*/

/********************************************************************************
** Form generated from reading UI file 'qsamplerOptionsForm.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_QSAMPLEROPTIONSFORM_H
#define UI_QSAMPLEROPTIONSFORM_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_qsamplerOptionsForm
{
public:
    QGridLayout *gridLayout;
    QTabWidget *OptionsTabWidget;
    QWidget *GeneralTabPage;
    QVBoxLayout *vboxLayout;
    QGroupBox *MessagesGroupBox;
    QHBoxLayout *hboxLayout;
    QLabel *MessagesFontTextLabel;
    QPushButton *MessagesFontPushButton;
    QSpacerItem *spacerItem;
    QCheckBox *MessagesLimitCheckBox;
    QSpinBox *MessagesLimitLinesSpinBox;
    QGroupBox *OtherGroupBox;
    QGridLayout *gridLayout1;
    QCheckBox *ConfirmRemoveCheckBox;
    QSpacerItem *spacerItem1;
    QLabel *MaxRecentFilesTextLabel;
    QSpinBox *MaxRecentFilesSpinBox;
    QCheckBox *KeepOnTopCheckBox;
    QCheckBox *StdoutCaptureCheckBox;
    QCheckBox *ConfirmResetCheckBox;
    QCheckBox *CompletePathCheckBox;
    QCheckBox *ConfirmRestartCheckBox;
    QCheckBox *InstrumentNamesCheckBox;
    QCheckBox *ConfirmErrorCheckBox;
    QSpacerItem *spacerItem2;
    QWidget *DisplayTabPage;
    QVBoxLayout *vboxLayout1;
    QGroupBox *ChannelsGroupBox;
    QGridLayout *gridLayout2;
    QLabel *DisplayFontTextLabel;
    QPushButton *DisplayFontPushButton;
    QSpacerItem *spacerItem3;
    QCheckBox *AutoRefreshCheckBox;
    QSpinBox *AutoRefreshTimeSpinBox;
    QCheckBox *DisplayEffectCheckBox;
    QGroupBox *DisplayCustomGroupBox;
    QGridLayout *gridLayout3;
    QLabel *CustomColorThemeTextLabel;
    QComboBox *CustomColorThemeComboBox;
    QToolButton *CustomColorThemeToolButton;
    QSpacerItem *spacerItem4;
    QLabel *CustomStyleThemeTextLabel;
    QComboBox *CustomStyleThemeComboBox;
    QSpacerItem *spacerItem5;
    QGroupBox *DisplayDefaultsGroupBox;
    QGridLayout *gridLayout4;
    QSpacerItem *spacerItem6;
    QLabel *BaseFontSizeTextLabel;
    QComboBox *BaseFontSizeComboBox;
    QWidget *ServerTabPage;
    QVBoxLayout *vboxLayout2;
    QGroupBox *SettingsGroupBox;
    QGridLayout *gridLayout5;
    QLabel *ServerHostTextLabel;
    QComboBox *ServerPortComboBox;
    QLabel *ServerPortTextLabel;
    QComboBox *ServerHostComboBox;
    QLabel *ServerCmdLineTextLabel;
    QCheckBox *ServerStartCheckBox;
    QComboBox *ServerCmdLineComboBox;
    QLabel *StartDelayTextLabel;
    QSpinBox *StartDelaySpinBox;
    QSpacerItem *spacerItem7;
    QSpacerItem *spacerItem8;
    QSpacerItem *spacerItem9;
    QSpinBox *ServerTimeoutSpinBox;
    QLabel *ServerTimeoutTextLabel;
    QSpacerItem *spacerItem10;
    QGroupBox *LoggingGroupBox;
    QGridLayout *gridLayout6;
    QComboBox *MessagesLogPathComboBox;
    QToolButton *MessagesLogPathToolButton;
    QCheckBox *MessagesLogCheckBox;
    QWidget *TuningTabPage;
    QVBoxLayout *vboxLayout3;
    QGroupBox *LimitsGroupBox;
    QGridLayout *gridLayout7;
    QLabel *MaxVolumeTextLabel;
    QSpinBox *MaxVolumeSpinBox;
    QSpacerItem *spacerItem11;
    QSpacerItem *spacerItem12;
    QLabel *MaxVoicesTextLabel;
    QSpinBox *MaxVoicesSpinBox;
    QSpacerItem *spacerItem13;
    QLabel *MaxDiskStreamsTextLabel;
    QSpinBox *MaxStreamsSpinBox;
    QSpacerItem *spacerItem14;
    QDialogButtonBox *DialogButtonBox;

    void setupUi(QDialog *qsamplerOptionsForm)
    {
        if (qsamplerOptionsForm->objectName().isEmpty())
            qsamplerOptionsForm->setObjectName("qsamplerOptionsForm");
        qsamplerOptionsForm->resize(520, 320);
        const QIcon icon = QIcon(QString::fromUtf8(":/images/qsampler.svg"));
        qsamplerOptionsForm->setWindowIcon(icon);
        qsamplerOptionsForm->setSizeGripEnabled(true);
        gridLayout = new QGridLayout(qsamplerOptionsForm);
        gridLayout->setSpacing(4);
        gridLayout->setContentsMargins(4, 4, 4, 4);
        gridLayout->setObjectName("gridLayout");
        OptionsTabWidget = new QTabWidget(qsamplerOptionsForm);
        OptionsTabWidget->setObjectName("OptionsTabWidget");
        OptionsTabWidget->setAcceptDrops(false);
        GeneralTabPage = new QWidget();
        GeneralTabPage->setObjectName("GeneralTabPage");
        vboxLayout = new QVBoxLayout(GeneralTabPage);
        vboxLayout->setSpacing(4);
        vboxLayout->setContentsMargins(8, 8, 8, 8);
        vboxLayout->setObjectName("vboxLayout");
        MessagesGroupBox = new QGroupBox(GeneralTabPage);
        MessagesGroupBox->setObjectName("MessagesGroupBox");
        QFont font;
        font.setBold(true);
        MessagesGroupBox->setFont(font);
        MessagesGroupBox->setFlat(true);
        hboxLayout = new QHBoxLayout(MessagesGroupBox);
        hboxLayout->setSpacing(4);
        hboxLayout->setContentsMargins(4, 4, 4, 4);
        hboxLayout->setObjectName("hboxLayout");
        MessagesFontTextLabel = new QLabel(MessagesGroupBox);
        MessagesFontTextLabel->setObjectName("MessagesFontTextLabel");
        QSizePolicy sizePolicy(QSizePolicy::Policy::MinimumExpanding, QSizePolicy::Policy::Minimum);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(MessagesFontTextLabel->sizePolicy().hasHeightForWidth());
        MessagesFontTextLabel->setSizePolicy(sizePolicy);
        MessagesFontTextLabel->setMinimumSize(QSize(180, 0));
        MessagesFontTextLabel->setMaximumSize(QSize(180, 32767));
        QFont font1;
        font1.setBold(false);
        MessagesFontTextLabel->setFont(font1);
        MessagesFontTextLabel->setAutoFillBackground(true);
        MessagesFontTextLabel->setFrameShape(QFrame::StyledPanel);
        MessagesFontTextLabel->setFrameShadow(QFrame::Sunken);
        MessagesFontTextLabel->setAlignment(Qt::AlignCenter);
        MessagesFontTextLabel->setWordWrap(false);

        hboxLayout->addWidget(MessagesFontTextLabel);

        MessagesFontPushButton = new QPushButton(MessagesGroupBox);
        MessagesFontPushButton->setObjectName("MessagesFontPushButton");
        MessagesFontPushButton->setFont(font1);
        MessagesFontPushButton->setAutoDefault(false);

        hboxLayout->addWidget(MessagesFontPushButton);

        spacerItem = new QSpacerItem(20, 8, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout->addItem(spacerItem);

        MessagesLimitCheckBox = new QCheckBox(MessagesGroupBox);
        MessagesLimitCheckBox->setObjectName("MessagesLimitCheckBox");
        MessagesLimitCheckBox->setFont(font1);

        hboxLayout->addWidget(MessagesLimitCheckBox);

        MessagesLimitLinesSpinBox = new QSpinBox(MessagesGroupBox);
        MessagesLimitLinesSpinBox->setObjectName("MessagesLimitLinesSpinBox");
        MessagesLimitLinesSpinBox->setFont(font1);
        MessagesLimitLinesSpinBox->setMinimum(100);
        MessagesLimitLinesSpinBox->setMaximum(10000);
        MessagesLimitLinesSpinBox->setSingleStep(100);
        MessagesLimitLinesSpinBox->setValue(1000);

        hboxLayout->addWidget(MessagesLimitLinesSpinBox);


        vboxLayout->addWidget(MessagesGroupBox);

        OtherGroupBox = new QGroupBox(GeneralTabPage);
        OtherGroupBox->setObjectName("OtherGroupBox");
        OtherGroupBox->setFont(font);
        OtherGroupBox->setFlat(true);
        gridLayout1 = new QGridLayout(OtherGroupBox);
        gridLayout1->setSpacing(4);
        gridLayout1->setContentsMargins(4, 4, 4, 4);
        gridLayout1->setObjectName("gridLayout1");
        ConfirmRemoveCheckBox = new QCheckBox(OtherGroupBox);
        ConfirmRemoveCheckBox->setObjectName("ConfirmRemoveCheckBox");
        ConfirmRemoveCheckBox->setFont(font1);

        gridLayout1->addWidget(ConfirmRemoveCheckBox, 0, 0, 1, 1);

        spacerItem1 = new QSpacerItem(20, 8, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout1->addItem(spacerItem1, 0, 1, 5, 1);

        MaxRecentFilesTextLabel = new QLabel(OtherGroupBox);
        MaxRecentFilesTextLabel->setObjectName("MaxRecentFilesTextLabel");
        MaxRecentFilesTextLabel->setFont(font1);
        MaxRecentFilesTextLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        MaxRecentFilesTextLabel->setWordWrap(false);

        gridLayout1->addWidget(MaxRecentFilesTextLabel, 0, 2, 1, 1);

        MaxRecentFilesSpinBox = new QSpinBox(OtherGroupBox);
        MaxRecentFilesSpinBox->setObjectName("MaxRecentFilesSpinBox");
        MaxRecentFilesSpinBox->setFont(font1);
        MaxRecentFilesSpinBox->setMinimum(0);
        MaxRecentFilesSpinBox->setMaximum(20);
        MaxRecentFilesSpinBox->setValue(5);

        gridLayout1->addWidget(MaxRecentFilesSpinBox, 0, 3, 1, 1);

        KeepOnTopCheckBox = new QCheckBox(OtherGroupBox);
        KeepOnTopCheckBox->setObjectName("KeepOnTopCheckBox");
        KeepOnTopCheckBox->setFont(font1);

        gridLayout1->addWidget(KeepOnTopCheckBox, 1, 0, 1, 1);

        StdoutCaptureCheckBox = new QCheckBox(OtherGroupBox);
        StdoutCaptureCheckBox->setObjectName("StdoutCaptureCheckBox");
        StdoutCaptureCheckBox->setFont(font1);

        gridLayout1->addWidget(StdoutCaptureCheckBox, 2, 0, 1, 1);

        ConfirmResetCheckBox = new QCheckBox(OtherGroupBox);
        ConfirmResetCheckBox->setObjectName("ConfirmResetCheckBox");
        ConfirmResetCheckBox->setFont(font1);

        gridLayout1->addWidget(ConfirmResetCheckBox, 2, 2, 1, 2);

        CompletePathCheckBox = new QCheckBox(OtherGroupBox);
        CompletePathCheckBox->setObjectName("CompletePathCheckBox");
        CompletePathCheckBox->setFont(font1);

        gridLayout1->addWidget(CompletePathCheckBox, 3, 0, 1, 1);

        ConfirmRestartCheckBox = new QCheckBox(OtherGroupBox);
        ConfirmRestartCheckBox->setObjectName("ConfirmRestartCheckBox");
        ConfirmRestartCheckBox->setFont(font1);

        gridLayout1->addWidget(ConfirmRestartCheckBox, 3, 2, 1, 2);

        InstrumentNamesCheckBox = new QCheckBox(OtherGroupBox);
        InstrumentNamesCheckBox->setObjectName("InstrumentNamesCheckBox");
        InstrumentNamesCheckBox->setFont(font1);

        gridLayout1->addWidget(InstrumentNamesCheckBox, 4, 0, 1, 1);

        ConfirmErrorCheckBox = new QCheckBox(OtherGroupBox);
        ConfirmErrorCheckBox->setObjectName("ConfirmErrorCheckBox");
        ConfirmErrorCheckBox->setFont(font1);

        gridLayout1->addWidget(ConfirmErrorCheckBox, 4, 2, 1, 2);


        vboxLayout->addWidget(OtherGroupBox);

        spacerItem2 = new QSpacerItem(8, 20, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(spacerItem2);

        OptionsTabWidget->addTab(GeneralTabPage, QString());
        DisplayTabPage = new QWidget();
        DisplayTabPage->setObjectName("DisplayTabPage");
        vboxLayout1 = new QVBoxLayout(DisplayTabPage);
        vboxLayout1->setSpacing(4);
        vboxLayout1->setContentsMargins(8, 8, 8, 8);
        vboxLayout1->setObjectName("vboxLayout1");
        ChannelsGroupBox = new QGroupBox(DisplayTabPage);
        ChannelsGroupBox->setObjectName("ChannelsGroupBox");
        ChannelsGroupBox->setFont(font);
        ChannelsGroupBox->setFlat(true);
        gridLayout2 = new QGridLayout(ChannelsGroupBox);
        gridLayout2->setSpacing(4);
        gridLayout2->setContentsMargins(4, 4, 4, 4);
        gridLayout2->setObjectName("gridLayout2");
        DisplayFontTextLabel = new QLabel(ChannelsGroupBox);
        DisplayFontTextLabel->setObjectName("DisplayFontTextLabel");
        sizePolicy.setHeightForWidth(DisplayFontTextLabel->sizePolicy().hasHeightForWidth());
        DisplayFontTextLabel->setSizePolicy(sizePolicy);
        DisplayFontTextLabel->setMinimumSize(QSize(180, 0));
        DisplayFontTextLabel->setMaximumSize(QSize(180, 32767));
        DisplayFontTextLabel->setFont(font1);
        DisplayFontTextLabel->setAutoFillBackground(true);
        DisplayFontTextLabel->setFrameShape(QFrame::StyledPanel);
        DisplayFontTextLabel->setFrameShadow(QFrame::Sunken);
        DisplayFontTextLabel->setLineWidth(1);
        DisplayFontTextLabel->setAlignment(Qt::AlignCenter);
        DisplayFontTextLabel->setWordWrap(false);

        gridLayout2->addWidget(DisplayFontTextLabel, 0, 0, 1, 1);

        DisplayFontPushButton = new QPushButton(ChannelsGroupBox);
        DisplayFontPushButton->setObjectName("DisplayFontPushButton");
        DisplayFontPushButton->setFont(font1);
        DisplayFontPushButton->setAutoDefault(false);

        gridLayout2->addWidget(DisplayFontPushButton, 0, 1, 1, 1);

        spacerItem3 = new QSpacerItem(20, 8, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout2->addItem(spacerItem3, 0, 2, 1, 1);

        AutoRefreshCheckBox = new QCheckBox(ChannelsGroupBox);
        AutoRefreshCheckBox->setObjectName("AutoRefreshCheckBox");
        AutoRefreshCheckBox->setFont(font1);

        gridLayout2->addWidget(AutoRefreshCheckBox, 0, 3, 1, 1);

        AutoRefreshTimeSpinBox = new QSpinBox(ChannelsGroupBox);
        AutoRefreshTimeSpinBox->setObjectName("AutoRefreshTimeSpinBox");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(AutoRefreshTimeSpinBox->sizePolicy().hasHeightForWidth());
        AutoRefreshTimeSpinBox->setSizePolicy(sizePolicy1);
        AutoRefreshTimeSpinBox->setFont(font1);
        AutoRefreshTimeSpinBox->setMinimum(200);
        AutoRefreshTimeSpinBox->setMaximum(20000);
        AutoRefreshTimeSpinBox->setSingleStep(100);
        AutoRefreshTimeSpinBox->setValue(1000);

        gridLayout2->addWidget(AutoRefreshTimeSpinBox, 0, 4, 1, 1);

        DisplayEffectCheckBox = new QCheckBox(ChannelsGroupBox);
        DisplayEffectCheckBox->setObjectName("DisplayEffectCheckBox");
        DisplayEffectCheckBox->setFont(font1);

        gridLayout2->addWidget(DisplayEffectCheckBox, 1, 0, 1, 2);


        vboxLayout1->addWidget(ChannelsGroupBox);

        DisplayCustomGroupBox = new QGroupBox(DisplayTabPage);
        DisplayCustomGroupBox->setObjectName("DisplayCustomGroupBox");
        DisplayCustomGroupBox->setFont(font);
        DisplayCustomGroupBox->setFlat(true);
        gridLayout3 = new QGridLayout(DisplayCustomGroupBox);
        gridLayout3->setSpacing(4);
        gridLayout3->setContentsMargins(8, 8, 8, 8);
        gridLayout3->setObjectName("gridLayout3");
        CustomColorThemeTextLabel = new QLabel(DisplayCustomGroupBox);
        CustomColorThemeTextLabel->setObjectName("CustomColorThemeTextLabel");
        CustomColorThemeTextLabel->setFont(font1);
        CustomColorThemeTextLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout3->addWidget(CustomColorThemeTextLabel, 0, 0, 1, 1);

        CustomColorThemeComboBox = new QComboBox(DisplayCustomGroupBox);
        CustomColorThemeComboBox->addItem(QString());
        CustomColorThemeComboBox->addItem(QString());
        CustomColorThemeComboBox->addItem(QString());
        CustomColorThemeComboBox->setObjectName("CustomColorThemeComboBox");
        CustomColorThemeComboBox->setFont(font1);
        CustomColorThemeComboBox->setEditable(false);

        gridLayout3->addWidget(CustomColorThemeComboBox, 0, 1, 1, 1);

        CustomColorThemeToolButton = new QToolButton(DisplayCustomGroupBox);
        CustomColorThemeToolButton->setObjectName("CustomColorThemeToolButton");
        CustomColorThemeToolButton->setMinimumSize(QSize(22, 22));
        CustomColorThemeToolButton->setMaximumSize(QSize(24, 24));
        CustomColorThemeToolButton->setFont(font1);
        CustomColorThemeToolButton->setFocusPolicy(Qt::TabFocus);

        gridLayout3->addWidget(CustomColorThemeToolButton, 0, 3, 1, 1);

        spacerItem4 = new QSpacerItem(20, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout3->addItem(spacerItem4, 0, 4, 1, 1);

        CustomStyleThemeTextLabel = new QLabel(DisplayCustomGroupBox);
        CustomStyleThemeTextLabel->setObjectName("CustomStyleThemeTextLabel");
        CustomStyleThemeTextLabel->setFont(font1);

        gridLayout3->addWidget(CustomStyleThemeTextLabel, 0, 5, 1, 1);

        CustomStyleThemeComboBox = new QComboBox(DisplayCustomGroupBox);
        CustomStyleThemeComboBox->addItem(QString());
        CustomStyleThemeComboBox->setObjectName("CustomStyleThemeComboBox");
        CustomStyleThemeComboBox->setFont(font1);
        CustomStyleThemeComboBox->setEditable(false);

        gridLayout3->addWidget(CustomStyleThemeComboBox, 0, 6, 1, 1);


        vboxLayout1->addWidget(DisplayCustomGroupBox);

        spacerItem5 = new QSpacerItem(8, 20, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout1->addItem(spacerItem5);

        DisplayDefaultsGroupBox = new QGroupBox(DisplayTabPage);
        DisplayDefaultsGroupBox->setObjectName("DisplayDefaultsGroupBox");
        DisplayDefaultsGroupBox->setFont(font);
        DisplayDefaultsGroupBox->setFlat(true);
        gridLayout4 = new QGridLayout(DisplayDefaultsGroupBox);
        gridLayout4->setSpacing(4);
        gridLayout4->setContentsMargins(4, 4, 4, 4);
        gridLayout4->setObjectName("gridLayout4");
        spacerItem6 = new QSpacerItem(20, 8, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout4->addItem(spacerItem6, 0, 0, 1, 1);

        BaseFontSizeTextLabel = new QLabel(DisplayDefaultsGroupBox);
        BaseFontSizeTextLabel->setObjectName("BaseFontSizeTextLabel");
        BaseFontSizeTextLabel->setFont(font1);
        BaseFontSizeTextLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout4->addWidget(BaseFontSizeTextLabel, 0, 1, 1, 1);

        BaseFontSizeComboBox = new QComboBox(DisplayDefaultsGroupBox);
        BaseFontSizeComboBox->addItem(QString());
        BaseFontSizeComboBox->addItem(QString());
        BaseFontSizeComboBox->addItem(QString());
        BaseFontSizeComboBox->addItem(QString());
        BaseFontSizeComboBox->addItem(QString());
        BaseFontSizeComboBox->addItem(QString());
        BaseFontSizeComboBox->addItem(QString());
        BaseFontSizeComboBox->addItem(QString());
        BaseFontSizeComboBox->setObjectName("BaseFontSizeComboBox");
        BaseFontSizeComboBox->setFont(font1);
        BaseFontSizeComboBox->setEditable(true);

        gridLayout4->addWidget(BaseFontSizeComboBox, 0, 2, 1, 1);


        vboxLayout1->addWidget(DisplayDefaultsGroupBox);

        OptionsTabWidget->addTab(DisplayTabPage, QString());
        ServerTabPage = new QWidget();
        ServerTabPage->setObjectName("ServerTabPage");
        vboxLayout2 = new QVBoxLayout(ServerTabPage);
        vboxLayout2->setSpacing(4);
        vboxLayout2->setContentsMargins(8, 8, 8, 8);
        vboxLayout2->setObjectName("vboxLayout2");
        SettingsGroupBox = new QGroupBox(ServerTabPage);
        SettingsGroupBox->setObjectName("SettingsGroupBox");
        SettingsGroupBox->setFont(font);
        SettingsGroupBox->setFlat(true);
        gridLayout5 = new QGridLayout(SettingsGroupBox);
        gridLayout5->setSpacing(4);
        gridLayout5->setContentsMargins(4, 4, 4, 4);
        gridLayout5->setObjectName("gridLayout5");
        gridLayout5->setContentsMargins(4, 4, 4, 4);
        ServerHostTextLabel = new QLabel(SettingsGroupBox);
        ServerHostTextLabel->setObjectName("ServerHostTextLabel");
        ServerHostTextLabel->setFont(font1);
        ServerHostTextLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        ServerHostTextLabel->setWordWrap(false);

        gridLayout5->addWidget(ServerHostTextLabel, 0, 0, 1, 1);

        ServerPortComboBox = new QComboBox(SettingsGroupBox);
        ServerPortComboBox->addItem(QString());
        ServerPortComboBox->setObjectName("ServerPortComboBox");
        ServerPortComboBox->setMaximumSize(QSize(60, 32767));
        ServerPortComboBox->setFont(font1);
        ServerPortComboBox->setEditable(true);

        gridLayout5->addWidget(ServerPortComboBox, 0, 4, 1, 1);

        ServerPortTextLabel = new QLabel(SettingsGroupBox);
        ServerPortTextLabel->setObjectName("ServerPortTextLabel");
        ServerPortTextLabel->setMaximumSize(QSize(60, 32767));
        ServerPortTextLabel->setFont(font1);
        ServerPortTextLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        ServerPortTextLabel->setWordWrap(false);

        gridLayout5->addWidget(ServerPortTextLabel, 0, 3, 1, 1);

        ServerHostComboBox = new QComboBox(SettingsGroupBox);
        ServerHostComboBox->addItem(QString());
        ServerHostComboBox->setObjectName("ServerHostComboBox");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(ServerHostComboBox->sizePolicy().hasHeightForWidth());
        ServerHostComboBox->setSizePolicy(sizePolicy2);
        ServerHostComboBox->setFont(font1);
        ServerHostComboBox->setEditable(true);

        gridLayout5->addWidget(ServerHostComboBox, 0, 1, 1, 2);

        ServerCmdLineTextLabel = new QLabel(SettingsGroupBox);
        ServerCmdLineTextLabel->setObjectName("ServerCmdLineTextLabel");
        ServerCmdLineTextLabel->setFont(font1);
        ServerCmdLineTextLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        ServerCmdLineTextLabel->setWordWrap(false);

        gridLayout5->addWidget(ServerCmdLineTextLabel, 4, 0, 1, 1);

        ServerStartCheckBox = new QCheckBox(SettingsGroupBox);
        ServerStartCheckBox->setObjectName("ServerStartCheckBox");
        ServerStartCheckBox->setFont(font1);

        gridLayout5->addWidget(ServerStartCheckBox, 3, 1, 1, 4);

        ServerCmdLineComboBox = new QComboBox(SettingsGroupBox);
        ServerCmdLineComboBox->addItem(QString());
        ServerCmdLineComboBox->setObjectName("ServerCmdLineComboBox");
        sizePolicy2.setHeightForWidth(ServerCmdLineComboBox->sizePolicy().hasHeightForWidth());
        ServerCmdLineComboBox->setSizePolicy(sizePolicy2);
        ServerCmdLineComboBox->setFont(font1);
        ServerCmdLineComboBox->setEditable(true);

        gridLayout5->addWidget(ServerCmdLineComboBox, 4, 1, 1, 4);

        StartDelayTextLabel = new QLabel(SettingsGroupBox);
        StartDelayTextLabel->setObjectName("StartDelayTextLabel");
        StartDelayTextLabel->setFont(font1);
        StartDelayTextLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        StartDelayTextLabel->setWordWrap(false);

        gridLayout5->addWidget(StartDelayTextLabel, 5, 0, 1, 1);

        StartDelaySpinBox = new QSpinBox(SettingsGroupBox);
        StartDelaySpinBox->setObjectName("StartDelaySpinBox");
        QSizePolicy sizePolicy3(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Fixed);
        sizePolicy3.setHorizontalStretch(0);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(StartDelaySpinBox->sizePolicy().hasHeightForWidth());
        StartDelaySpinBox->setSizePolicy(sizePolicy3);
        StartDelaySpinBox->setMinimumSize(QSize(40, 0));
        StartDelaySpinBox->setFont(font1);
        StartDelaySpinBox->setMinimum(1);
        StartDelaySpinBox->setMaximum(100);
        StartDelaySpinBox->setValue(3);

        gridLayout5->addWidget(StartDelaySpinBox, 5, 1, 1, 1);

        spacerItem7 = new QSpacerItem(20, 8, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout5->addItem(spacerItem7, 5, 2, 1, 3);

        spacerItem8 = new QSpacerItem(20, 8, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout5->addItem(spacerItem8, 2, 1, 1, 1);

        spacerItem9 = new QSpacerItem(20, 8, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout5->addItem(spacerItem9, 1, 2, 1, 3);

        ServerTimeoutSpinBox = new QSpinBox(SettingsGroupBox);
        ServerTimeoutSpinBox->setObjectName("ServerTimeoutSpinBox");
        sizePolicy3.setHeightForWidth(ServerTimeoutSpinBox->sizePolicy().hasHeightForWidth());
        ServerTimeoutSpinBox->setSizePolicy(sizePolicy3);
        ServerTimeoutSpinBox->setMinimumSize(QSize(40, 0));
        ServerTimeoutSpinBox->setFont(font1);
        ServerTimeoutSpinBox->setMinimum(100);
        ServerTimeoutSpinBox->setMaximum(60000);
        ServerTimeoutSpinBox->setSingleStep(100);
        ServerTimeoutSpinBox->setValue(1000);

        gridLayout5->addWidget(ServerTimeoutSpinBox, 1, 1, 1, 1);

        ServerTimeoutTextLabel = new QLabel(SettingsGroupBox);
        ServerTimeoutTextLabel->setObjectName("ServerTimeoutTextLabel");
        ServerTimeoutTextLabel->setFont(font1);
        ServerTimeoutTextLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        ServerTimeoutTextLabel->setWordWrap(false);

        gridLayout5->addWidget(ServerTimeoutTextLabel, 1, 0, 1, 1);

        spacerItem10 = new QSpacerItem(8, 20, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout5->addItem(spacerItem10, 6, 1, 1, 1);


        vboxLayout2->addWidget(SettingsGroupBox);

        LoggingGroupBox = new QGroupBox(ServerTabPage);
        LoggingGroupBox->setObjectName("LoggingGroupBox");
        LoggingGroupBox->setFont(font);
        LoggingGroupBox->setFlat(true);
        gridLayout6 = new QGridLayout(LoggingGroupBox);
        gridLayout6->setSpacing(4);
        gridLayout6->setContentsMargins(4, 4, 4, 4);
        gridLayout6->setObjectName("gridLayout6");
        gridLayout6->setContentsMargins(8, 8, 8, 8);
        MessagesLogPathComboBox = new QComboBox(LoggingGroupBox);
        MessagesLogPathComboBox->setObjectName("MessagesLogPathComboBox");
        sizePolicy2.setHeightForWidth(MessagesLogPathComboBox->sizePolicy().hasHeightForWidth());
        MessagesLogPathComboBox->setSizePolicy(sizePolicy2);
        MessagesLogPathComboBox->setFont(font1);
        MessagesLogPathComboBox->setEditable(true);

        gridLayout6->addWidget(MessagesLogPathComboBox, 0, 1, 1, 3);

        MessagesLogPathToolButton = new QToolButton(LoggingGroupBox);
        MessagesLogPathToolButton->setObjectName("MessagesLogPathToolButton");
        MessagesLogPathToolButton->setMinimumSize(QSize(22, 22));
        MessagesLogPathToolButton->setMaximumSize(QSize(24, 24));
        MessagesLogPathToolButton->setFont(font1);
        MessagesLogPathToolButton->setFocusPolicy(Qt::TabFocus);

        gridLayout6->addWidget(MessagesLogPathToolButton, 0, 4, 1, 1);

        MessagesLogCheckBox = new QCheckBox(LoggingGroupBox);
        MessagesLogCheckBox->setObjectName("MessagesLogCheckBox");
        MessagesLogCheckBox->setFont(font1);

        gridLayout6->addWidget(MessagesLogCheckBox, 0, 0, 1, 1);


        vboxLayout2->addWidget(LoggingGroupBox);

        OptionsTabWidget->addTab(ServerTabPage, QString());
        TuningTabPage = new QWidget();
        TuningTabPage->setObjectName("TuningTabPage");
        vboxLayout3 = new QVBoxLayout(TuningTabPage);
        vboxLayout3->setSpacing(4);
        vboxLayout3->setContentsMargins(8, 8, 8, 8);
        vboxLayout3->setObjectName("vboxLayout3");
        LimitsGroupBox = new QGroupBox(TuningTabPage);
        LimitsGroupBox->setObjectName("LimitsGroupBox");
        LimitsGroupBox->setFont(font);
        LimitsGroupBox->setFlat(true);
        gridLayout7 = new QGridLayout(LimitsGroupBox);
        gridLayout7->setSpacing(4);
        gridLayout7->setContentsMargins(4, 4, 4, 4);
        gridLayout7->setObjectName("gridLayout7");
        MaxVolumeTextLabel = new QLabel(LimitsGroupBox);
        MaxVolumeTextLabel->setObjectName("MaxVolumeTextLabel");
        MaxVolumeTextLabel->setFont(font1);
        MaxVolumeTextLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        MaxVolumeTextLabel->setWordWrap(false);

        gridLayout7->addWidget(MaxVolumeTextLabel, 0, 0, 1, 1);

        MaxVolumeSpinBox = new QSpinBox(LimitsGroupBox);
        MaxVolumeSpinBox->setObjectName("MaxVolumeSpinBox");
        sizePolicy1.setHeightForWidth(MaxVolumeSpinBox->sizePolicy().hasHeightForWidth());
        MaxVolumeSpinBox->setSizePolicy(sizePolicy1);
        MaxVolumeSpinBox->setFont(font1);
        MaxVolumeSpinBox->setMaximum(1000);
        MaxVolumeSpinBox->setSingleStep(10);
        MaxVolumeSpinBox->setValue(100);

        gridLayout7->addWidget(MaxVolumeSpinBox, 0, 1, 1, 1);

        spacerItem11 = new QSpacerItem(20, 8, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout7->addItem(spacerItem11, 0, 2, 1, 2);

        spacerItem12 = new QSpacerItem(20, 8, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout7->addItem(spacerItem12, 1, 0, 1, 4);

        MaxVoicesTextLabel = new QLabel(LimitsGroupBox);
        MaxVoicesTextLabel->setObjectName("MaxVoicesTextLabel");
        MaxVoicesTextLabel->setFont(font1);

        gridLayout7->addWidget(MaxVoicesTextLabel, 2, 0, 1, 2);

        MaxVoicesSpinBox = new QSpinBox(LimitsGroupBox);
        MaxVoicesSpinBox->setObjectName("MaxVoicesSpinBox");
        MaxVoicesSpinBox->setMinimumSize(QSize(60, 0));
        MaxVoicesSpinBox->setFont(font1);
        MaxVoicesSpinBox->setMaximum(999999999);

        gridLayout7->addWidget(MaxVoicesSpinBox, 2, 2, 1, 1);

        spacerItem13 = new QSpacerItem(20, 8, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout7->addItem(spacerItem13, 2, 3, 2, 1);

        MaxDiskStreamsTextLabel = new QLabel(LimitsGroupBox);
        MaxDiskStreamsTextLabel->setObjectName("MaxDiskStreamsTextLabel");
        MaxDiskStreamsTextLabel->setFont(font1);

        gridLayout7->addWidget(MaxDiskStreamsTextLabel, 3, 0, 1, 2);

        MaxStreamsSpinBox = new QSpinBox(LimitsGroupBox);
        MaxStreamsSpinBox->setObjectName("MaxStreamsSpinBox");
        MaxStreamsSpinBox->setMinimumSize(QSize(60, 0));
        MaxStreamsSpinBox->setFont(font1);
        MaxStreamsSpinBox->setMaximum(999999999);

        gridLayout7->addWidget(MaxStreamsSpinBox, 3, 2, 1, 1);


        vboxLayout3->addWidget(LimitsGroupBox);

        spacerItem14 = new QSpacerItem(20, 20, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout3->addItem(spacerItem14);

        OptionsTabWidget->addTab(TuningTabPage, QString());

        gridLayout->addWidget(OptionsTabWidget, 0, 0, 1, 1);

        DialogButtonBox = new QDialogButtonBox(qsamplerOptionsForm);
        DialogButtonBox->setObjectName("DialogButtonBox");
        DialogButtonBox->setOrientation(Qt::Horizontal);
        DialogButtonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        gridLayout->addWidget(DialogButtonBox, 1, 0, 1, 1);

        DialogButtonBox->raise();
        OptionsTabWidget->raise();
#if QT_CONFIG(shortcut)
        MaxRecentFilesTextLabel->setBuddy(MaxRecentFilesSpinBox);
        CustomColorThemeTextLabel->setBuddy(CustomColorThemeComboBox);
        CustomStyleThemeTextLabel->setBuddy(CustomStyleThemeComboBox);
        BaseFontSizeTextLabel->setBuddy(BaseFontSizeComboBox);
        ServerHostTextLabel->setBuddy(ServerHostComboBox);
        ServerPortTextLabel->setBuddy(ServerPortComboBox);
        ServerCmdLineTextLabel->setBuddy(ServerCmdLineComboBox);
        StartDelayTextLabel->setBuddy(StartDelaySpinBox);
        ServerTimeoutTextLabel->setBuddy(ServerTimeoutSpinBox);
        MaxVolumeTextLabel->setBuddy(MaxVolumeSpinBox);
        MaxVoicesTextLabel->setBuddy(MaxVoicesSpinBox);
        MaxDiskStreamsTextLabel->setBuddy(MaxStreamsSpinBox);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(OptionsTabWidget, MessagesFontPushButton);
        QWidget::setTabOrder(MessagesFontPushButton, MessagesLimitCheckBox);
        QWidget::setTabOrder(MessagesLimitCheckBox, MessagesLimitLinesSpinBox);
        QWidget::setTabOrder(MessagesLimitLinesSpinBox, ConfirmRemoveCheckBox);
        QWidget::setTabOrder(ConfirmRemoveCheckBox, MaxRecentFilesSpinBox);
        QWidget::setTabOrder(MaxRecentFilesSpinBox, KeepOnTopCheckBox);
        QWidget::setTabOrder(KeepOnTopCheckBox, StdoutCaptureCheckBox);
        QWidget::setTabOrder(StdoutCaptureCheckBox, ConfirmResetCheckBox);
        QWidget::setTabOrder(ConfirmResetCheckBox, CompletePathCheckBox);
        QWidget::setTabOrder(CompletePathCheckBox, ConfirmRestartCheckBox);
        QWidget::setTabOrder(ConfirmRestartCheckBox, InstrumentNamesCheckBox);
        QWidget::setTabOrder(InstrumentNamesCheckBox, ConfirmErrorCheckBox);
        QWidget::setTabOrder(ConfirmErrorCheckBox, DisplayFontPushButton);
        QWidget::setTabOrder(DisplayFontPushButton, AutoRefreshCheckBox);
        QWidget::setTabOrder(AutoRefreshCheckBox, AutoRefreshTimeSpinBox);
        QWidget::setTabOrder(AutoRefreshTimeSpinBox, DisplayEffectCheckBox);
        QWidget::setTabOrder(DisplayEffectCheckBox, CustomColorThemeComboBox);
        QWidget::setTabOrder(CustomColorThemeComboBox, CustomColorThemeToolButton);
        QWidget::setTabOrder(CustomColorThemeToolButton, CustomStyleThemeComboBox);
        QWidget::setTabOrder(CustomStyleThemeComboBox, BaseFontSizeComboBox);
        QWidget::setTabOrder(BaseFontSizeComboBox, ServerPortComboBox);
        QWidget::setTabOrder(ServerPortComboBox, ServerHostComboBox);
        QWidget::setTabOrder(ServerHostComboBox, ServerStartCheckBox);
        QWidget::setTabOrder(ServerStartCheckBox, ServerCmdLineComboBox);
        QWidget::setTabOrder(ServerCmdLineComboBox, StartDelaySpinBox);
        QWidget::setTabOrder(StartDelaySpinBox, ServerTimeoutSpinBox);
        QWidget::setTabOrder(ServerTimeoutSpinBox, MessagesLogPathComboBox);
        QWidget::setTabOrder(MessagesLogPathComboBox, MessagesLogPathToolButton);
        QWidget::setTabOrder(MessagesLogPathToolButton, MessagesLogCheckBox);
        QWidget::setTabOrder(MessagesLogCheckBox, MaxVolumeSpinBox);
        QWidget::setTabOrder(MaxVolumeSpinBox, MaxVoicesSpinBox);
        QWidget::setTabOrder(MaxVoicesSpinBox, MaxStreamsSpinBox);
        QWidget::setTabOrder(MaxStreamsSpinBox, DialogButtonBox);

        retranslateUi(qsamplerOptionsForm);

        OptionsTabWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(qsamplerOptionsForm);
    } // setupUi

    void retranslateUi(QDialog *qsamplerOptionsForm)
    {
        qsamplerOptionsForm->setWindowTitle(QCoreApplication::translate("qsamplerOptionsForm", "Options", nullptr));
        MessagesGroupBox->setTitle(QCoreApplication::translate("qsamplerOptionsForm", "Messages", nullptr));
#if QT_CONFIG(tooltip)
        MessagesFontTextLabel->setToolTip(QCoreApplication::translate("qsamplerOptionsForm", "Sample messages text font display", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        MessagesFontPushButton->setToolTip(QCoreApplication::translate("qsamplerOptionsForm", "Select font for the messages text display", nullptr));
#endif // QT_CONFIG(tooltip)
        MessagesFontPushButton->setText(QCoreApplication::translate("qsamplerOptionsForm", "&Font...", nullptr));
#if QT_CONFIG(tooltip)
        MessagesLimitCheckBox->setToolTip(QCoreApplication::translate("qsamplerOptionsForm", "Whether to keep a maximum number of lines in the messages window", nullptr));
#endif // QT_CONFIG(tooltip)
        MessagesLimitCheckBox->setText(QCoreApplication::translate("qsamplerOptionsForm", "&Messages limit:", nullptr));
#if QT_CONFIG(tooltip)
        MessagesLimitLinesSpinBox->setToolTip(QCoreApplication::translate("qsamplerOptionsForm", "The maximum number of message lines to keep in view", nullptr));
#endif // QT_CONFIG(tooltip)
        MessagesLimitLinesSpinBox->setSuffix(QCoreApplication::translate("qsamplerOptionsForm", " lines", nullptr));
        OtherGroupBox->setTitle(QCoreApplication::translate("qsamplerOptionsForm", "Other", nullptr));
#if QT_CONFIG(tooltip)
        ConfirmRemoveCheckBox->setToolTip(QCoreApplication::translate("qsamplerOptionsForm", "Whether to ask for confirmation on removals", nullptr));
#endif // QT_CONFIG(tooltip)
        ConfirmRemoveCheckBox->setText(QCoreApplication::translate("qsamplerOptionsForm", "&Confirm removals", nullptr));
        MaxRecentFilesTextLabel->setText(QCoreApplication::translate("qsamplerOptionsForm", "&Number of recent files:", nullptr));
#if QT_CONFIG(tooltip)
        MaxRecentFilesSpinBox->setToolTip(QCoreApplication::translate("qsamplerOptionsForm", "The maximum number of recent files to keep in menu", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        KeepOnTopCheckBox->setToolTip(QCoreApplication::translate("qsamplerOptionsForm", "Whether to keep all child windows on top of the main window", nullptr));
#endif // QT_CONFIG(tooltip)
        KeepOnTopCheckBox->setText(QCoreApplication::translate("qsamplerOptionsForm", "&Keep child windows always on top", nullptr));
#if QT_CONFIG(tooltip)
        StdoutCaptureCheckBox->setToolTip(QCoreApplication::translate("qsamplerOptionsForm", "Whether to capture standard output (stdout/stderr) into messages window", nullptr));
#endif // QT_CONFIG(tooltip)
        StdoutCaptureCheckBox->setText(QCoreApplication::translate("qsamplerOptionsForm", "Capture standard &output", nullptr));
#if QT_CONFIG(tooltip)
        ConfirmResetCheckBox->setToolTip(QCoreApplication::translate("qsamplerOptionsForm", "Whether to ask for session reset", nullptr));
#endif // QT_CONFIG(tooltip)
        ConfirmResetCheckBox->setText(QCoreApplication::translate("qsamplerOptionsForm", "&Confirm session reset", nullptr));
#if QT_CONFIG(tooltip)
        CompletePathCheckBox->setToolTip(QCoreApplication::translate("qsamplerOptionsForm", "Whether to show the complete directory path of loaded session files", nullptr));
#endif // QT_CONFIG(tooltip)
        CompletePathCheckBox->setText(QCoreApplication::translate("qsamplerOptionsForm", "Show complete &path of session files", nullptr));
#if QT_CONFIG(tooltip)
        ConfirmRestartCheckBox->setToolTip(QCoreApplication::translate("qsamplerOptionsForm", "Whether to ask for session restart", nullptr));
#endif // QT_CONFIG(tooltip)
        ConfirmRestartCheckBox->setText(QCoreApplication::translate("qsamplerOptionsForm", "&Confirm session restart", nullptr));
#if QT_CONFIG(tooltip)
        InstrumentNamesCheckBox->setToolTip(QCoreApplication::translate("qsamplerOptionsForm", "Whether to show the actual instrument names as read from instrument file (using libgig)", nullptr));
#endif // QT_CONFIG(tooltip)
        InstrumentNamesCheckBox->setText(QCoreApplication::translate("qsamplerOptionsForm", "Show actual &instrument names", nullptr));
#if QT_CONFIG(tooltip)
        ConfirmErrorCheckBox->setToolTip(QCoreApplication::translate("qsamplerOptionsForm", "Whether to show session errors", nullptr));
#endif // QT_CONFIG(tooltip)
        ConfirmErrorCheckBox->setText(QCoreApplication::translate("qsamplerOptionsForm", "&Confirm session errors", nullptr));
        OptionsTabWidget->setTabText(OptionsTabWidget->indexOf(GeneralTabPage), QCoreApplication::translate("qsamplerOptionsForm", "&General", nullptr));
        ChannelsGroupBox->setTitle(QCoreApplication::translate("qsamplerOptionsForm", "Channels", nullptr));
#if QT_CONFIG(tooltip)
        DisplayFontTextLabel->setToolTip(QCoreApplication::translate("qsamplerOptionsForm", "Sample channel display font display", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        DisplayFontPushButton->setToolTip(QCoreApplication::translate("qsamplerOptionsForm", "Select font for the channel display", nullptr));
#endif // QT_CONFIG(tooltip)
        DisplayFontPushButton->setText(QCoreApplication::translate("qsamplerOptionsForm", "&Font...", nullptr));
#if QT_CONFIG(tooltip)
        AutoRefreshCheckBox->setToolTip(QCoreApplication::translate("qsamplerOptionsForm", "Whether to refresh the channels view automatically", nullptr));
#endif // QT_CONFIG(tooltip)
        AutoRefreshCheckBox->setText(QCoreApplication::translate("qsamplerOptionsForm", "&Auto refresh:", nullptr));
#if QT_CONFIG(tooltip)
        AutoRefreshTimeSpinBox->setToolTip(QCoreApplication::translate("qsamplerOptionsForm", "Time in milliseconds between each auto-refresh cycle", nullptr));
#endif // QT_CONFIG(tooltip)
        AutoRefreshTimeSpinBox->setSuffix(QCoreApplication::translate("qsamplerOptionsForm", " msec", nullptr));
#if QT_CONFIG(tooltip)
        DisplayEffectCheckBox->setToolTip(QCoreApplication::translate("qsamplerOptionsForm", "Whether to enable a shiny glass light effect on the channel display", nullptr));
#endif // QT_CONFIG(tooltip)
        DisplayEffectCheckBox->setText(QCoreApplication::translate("qsamplerOptionsForm", "Display shiny glass light &effect", nullptr));
        DisplayCustomGroupBox->setTitle(QCoreApplication::translate("qsamplerOptionsForm", "Custom", nullptr));
        CustomColorThemeTextLabel->setText(QCoreApplication::translate("qsamplerOptionsForm", "&Color palette theme:", nullptr));
        CustomColorThemeComboBox->setItemText(0, QCoreApplication::translate("qsamplerOptionsForm", "(default)", nullptr));
        CustomColorThemeComboBox->setItemText(1, QCoreApplication::translate("qsamplerOptionsForm", "Wonton Soup", nullptr));
        CustomColorThemeComboBox->setItemText(2, QCoreApplication::translate("qsamplerOptionsForm", "KXStudio", nullptr));

#if QT_CONFIG(tooltip)
        CustomColorThemeComboBox->setToolTip(QCoreApplication::translate("qsamplerOptionsForm", "Custom color palette theme", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        CustomColorThemeToolButton->setToolTip(QCoreApplication::translate("qsamplerOptionsForm", "Manage custom color palette themes", nullptr));
#endif // QT_CONFIG(tooltip)
        CustomColorThemeToolButton->setText(QCoreApplication::translate("qsamplerOptionsForm", "...", nullptr));
        CustomStyleThemeTextLabel->setText(QCoreApplication::translate("qsamplerOptionsForm", "&Widget style theme:", nullptr));
        CustomStyleThemeComboBox->setItemText(0, QCoreApplication::translate("qsamplerOptionsForm", "(default)", nullptr));

#if QT_CONFIG(tooltip)
        CustomStyleThemeComboBox->setToolTip(QCoreApplication::translate("qsamplerOptionsForm", "Custom widget style theme", nullptr));
#endif // QT_CONFIG(tooltip)
        DisplayDefaultsGroupBox->setTitle(QCoreApplication::translate("qsamplerOptionsForm", "Defaults", nullptr));
        BaseFontSizeTextLabel->setText(QCoreApplication::translate("qsamplerOptionsForm", "&Base font size:", nullptr));
        BaseFontSizeComboBox->setItemText(0, QCoreApplication::translate("qsamplerOptionsForm", "(default)", nullptr));
        BaseFontSizeComboBox->setItemText(1, QCoreApplication::translate("qsamplerOptionsForm", "6", nullptr));
        BaseFontSizeComboBox->setItemText(2, QCoreApplication::translate("qsamplerOptionsForm", "7", nullptr));
        BaseFontSizeComboBox->setItemText(3, QCoreApplication::translate("qsamplerOptionsForm", "8", nullptr));
        BaseFontSizeComboBox->setItemText(4, QCoreApplication::translate("qsamplerOptionsForm", "9", nullptr));
        BaseFontSizeComboBox->setItemText(5, QCoreApplication::translate("qsamplerOptionsForm", "10", nullptr));
        BaseFontSizeComboBox->setItemText(6, QCoreApplication::translate("qsamplerOptionsForm", "11", nullptr));
        BaseFontSizeComboBox->setItemText(7, QCoreApplication::translate("qsamplerOptionsForm", "12", nullptr));

#if QT_CONFIG(tooltip)
        BaseFontSizeComboBox->setToolTip(QCoreApplication::translate("qsamplerOptionsForm", "Base application font size (pt.)", nullptr));
#endif // QT_CONFIG(tooltip)
        OptionsTabWidget->setTabText(OptionsTabWidget->indexOf(DisplayTabPage), QCoreApplication::translate("qsamplerOptionsForm", "&Display", nullptr));
        SettingsGroupBox->setTitle(QCoreApplication::translate("qsamplerOptionsForm", "Settings", nullptr));
        ServerHostTextLabel->setText(QCoreApplication::translate("qsamplerOptionsForm", "&Host:", nullptr));
        ServerPortComboBox->setItemText(0, QCoreApplication::translate("qsamplerOptionsForm", "8888", nullptr));

#if QT_CONFIG(tooltip)
        ServerPortComboBox->setToolTip(QCoreApplication::translate("qsamplerOptionsForm", "LinuxSampler server listener port number", nullptr));
#endif // QT_CONFIG(tooltip)
        ServerPortTextLabel->setText(QCoreApplication::translate("qsamplerOptionsForm", "&Port:", nullptr));
        ServerHostComboBox->setItemText(0, QCoreApplication::translate("qsamplerOptionsForm", "localhost", nullptr));

#if QT_CONFIG(tooltip)
        ServerHostComboBox->setToolTip(QCoreApplication::translate("qsamplerOptionsForm", "LinuxSampler server host name or address", nullptr));
#endif // QT_CONFIG(tooltip)
        ServerCmdLineTextLabel->setText(QCoreApplication::translate("qsamplerOptionsForm", "&Command line:", nullptr));
#if QT_CONFIG(tooltip)
        ServerStartCheckBox->setToolTip(QCoreApplication::translate("qsamplerOptionsForm", "Whether to start the LinuxSampler server on local machine", nullptr));
#endif // QT_CONFIG(tooltip)
        ServerStartCheckBox->setText(QCoreApplication::translate("qsamplerOptionsForm", "&Start server locally", nullptr));
        ServerCmdLineComboBox->setItemText(0, QCoreApplication::translate("qsamplerOptionsForm", "linuxsampler", nullptr));

#if QT_CONFIG(tooltip)
        ServerCmdLineComboBox->setToolTip(QCoreApplication::translate("qsamplerOptionsForm", "Command line to start LinuxSampler server locally", nullptr));
#endif // QT_CONFIG(tooltip)
        StartDelayTextLabel->setText(QCoreApplication::translate("qsamplerOptionsForm", "Start &delay:", nullptr));
#if QT_CONFIG(tooltip)
        StartDelaySpinBox->setToolTip(QCoreApplication::translate("qsamplerOptionsForm", "Delay time in seconds after server startup", nullptr));
#endif // QT_CONFIG(tooltip)
        StartDelaySpinBox->setSuffix(QCoreApplication::translate("qsamplerOptionsForm", " secs", nullptr));
#if QT_CONFIG(tooltip)
        ServerTimeoutSpinBox->setToolTip(QCoreApplication::translate("qsamplerOptionsForm", "Receive timeout in milliseconds", nullptr));
#endif // QT_CONFIG(tooltip)
        ServerTimeoutSpinBox->setSuffix(QCoreApplication::translate("qsamplerOptionsForm", " msec", nullptr));
        ServerTimeoutTextLabel->setText(QCoreApplication::translate("qsamplerOptionsForm", "&Timeout:", nullptr));
        LoggingGroupBox->setTitle(QCoreApplication::translate("qsamplerOptionsForm", "Logging", nullptr));
#if QT_CONFIG(tooltip)
        MessagesLogPathComboBox->setToolTip(QCoreApplication::translate("qsamplerOptionsForm", "Messages log file", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        MessagesLogPathToolButton->setToolTip(QCoreApplication::translate("qsamplerOptionsForm", "Browse for the messages log file location", nullptr));
#endif // QT_CONFIG(tooltip)
        MessagesLogPathToolButton->setText(QCoreApplication::translate("qsamplerOptionsForm", "...", nullptr));
#if QT_CONFIG(tooltip)
        MessagesLogCheckBox->setToolTip(QCoreApplication::translate("qsamplerOptionsForm", "Whether to activate a messages logging to file.", nullptr));
#endif // QT_CONFIG(tooltip)
        MessagesLogCheckBox->setText(QCoreApplication::translate("qsamplerOptionsForm", "&Messages log file:", nullptr));
        OptionsTabWidget->setTabText(OptionsTabWidget->indexOf(ServerTabPage), QCoreApplication::translate("qsamplerOptionsForm", "&Server", nullptr));
        LimitsGroupBox->setTitle(QCoreApplication::translate("qsamplerOptionsForm", "Limits", nullptr));
        MaxVolumeTextLabel->setText(QCoreApplication::translate("qsamplerOptionsForm", "Maximum &volume:", nullptr));
#if QT_CONFIG(tooltip)
        MaxVolumeSpinBox->setToolTip(QCoreApplication::translate("qsamplerOptionsForm", "Upper limit for the sampler channel volume setting", nullptr));
#endif // QT_CONFIG(tooltip)
        MaxVolumeSpinBox->setSuffix(QCoreApplication::translate("qsamplerOptionsForm", " %", nullptr));
        MaxVoicesTextLabel->setText(QCoreApplication::translate("qsamplerOptionsForm", "Maximum &number of voices:", nullptr));
#if QT_CONFIG(tooltip)
        MaxVoicesSpinBox->setToolTip(QCoreApplication::translate("qsamplerOptionsForm", "Maximum number of voices", nullptr));
#endif // QT_CONFIG(tooltip)
        MaxDiskStreamsTextLabel->setText(QCoreApplication::translate("qsamplerOptionsForm", "Maximum number of disk &streams:", nullptr));
#if QT_CONFIG(tooltip)
        MaxStreamsSpinBox->setToolTip(QCoreApplication::translate("qsamplerOptionsForm", "Maximum number of disk streams", nullptr));
#endif // QT_CONFIG(tooltip)
        OptionsTabWidget->setTabText(OptionsTabWidget->indexOf(TuningTabPage), QCoreApplication::translate("qsamplerOptionsForm", "&Tuning", nullptr));
    } // retranslateUi

};

namespace Ui {
    class qsamplerOptionsForm: public Ui_qsamplerOptionsForm {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_QSAMPLEROPTIONSFORM_H
