/*


   Copyright (C) 2010-2021, rncbc aka Rui Nuno Capela. All rights reserved.
   Copyright (C) 2008, Christian Schoenebeck

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

 
*/

/********************************************************************************
** Form generated from reading UI file 'qsamplerChannelFxForm.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_QSAMPLERCHANNELFXFORM_H
#define UI_QSAMPLERCHANNELFXFORM_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QListView>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QTableView>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_qsamplerChannelFxForm
{
public:
    QVBoxLayout *vboxLayout;
    QHBoxLayout *hboxLayout;
    QGroupBox *groupBox;
    QVBoxLayout *vboxLayout1;
    QListView *SendsListView;
    QHBoxLayout *hboxLayout1;
    QSpacerItem *spacerItem;
    QPushButton *createPushButton;
    QPushButton *destroyPushButton;
    QVBoxLayout *vboxLayout2;
    QGroupBox *mainParametersGroupBox;
    QGridLayout *gridLayout;
    QLabel *label;
    QComboBox *depthCtrlComboBox;
    QLabel *label_2;
    QSpinBox *depthSpinBox;
    QGroupBox *audioRoutingGroupBox;
    QHBoxLayout *hboxLayout2;
    QTableView *audioRoutingTable;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *qsamplerChannelFxForm)
    {
        if (qsamplerChannelFxForm->objectName().isEmpty())
            qsamplerChannelFxForm->setObjectName("qsamplerChannelFxForm");
        qsamplerChannelFxForm->resize(518, 370);
        vboxLayout = new QVBoxLayout(qsamplerChannelFxForm);
        vboxLayout->setObjectName("vboxLayout");
        hboxLayout = new QHBoxLayout();
        hboxLayout->setObjectName("hboxLayout");
        groupBox = new QGroupBox(qsamplerChannelFxForm);
        groupBox->setObjectName("groupBox");
        vboxLayout1 = new QVBoxLayout(groupBox);
        vboxLayout1->setObjectName("vboxLayout1");
        SendsListView = new QListView(groupBox);
        SendsListView->setObjectName("SendsListView");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(SendsListView->sizePolicy().hasHeightForWidth());
        SendsListView->setSizePolicy(sizePolicy);
        SendsListView->setAlternatingRowColors(true);

        vboxLayout1->addWidget(SendsListView);

        hboxLayout1 = new QHBoxLayout();
        hboxLayout1->setObjectName("hboxLayout1");
        spacerItem = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout1->addItem(spacerItem);

        createPushButton = new QPushButton(groupBox);
        createPushButton->setObjectName("createPushButton");
        const QIcon icon = QIcon(QString::fromUtf8(":/images/itemNew.png"));
        createPushButton->setIcon(icon);

        hboxLayout1->addWidget(createPushButton);

        destroyPushButton = new QPushButton(groupBox);
        destroyPushButton->setObjectName("destroyPushButton");
        const QIcon icon1 = QIcon(QString::fromUtf8(":/images/formRemove.png"));
        destroyPushButton->setIcon(icon1);

        hboxLayout1->addWidget(destroyPushButton);


        vboxLayout1->addLayout(hboxLayout1);


        hboxLayout->addWidget(groupBox);

        vboxLayout2 = new QVBoxLayout();
        vboxLayout2->setObjectName("vboxLayout2");
        mainParametersGroupBox = new QGroupBox(qsamplerChannelFxForm);
        mainParametersGroupBox->setObjectName("mainParametersGroupBox");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Minimum);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(mainParametersGroupBox->sizePolicy().hasHeightForWidth());
        mainParametersGroupBox->setSizePolicy(sizePolicy1);
        gridLayout = new QGridLayout(mainParametersGroupBox);
        gridLayout->setObjectName("gridLayout");
        label = new QLabel(mainParametersGroupBox);
        label->setObjectName("label");

        gridLayout->addWidget(label, 0, 0, 1, 1);

        depthCtrlComboBox = new QComboBox(mainParametersGroupBox);
        depthCtrlComboBox->setObjectName("depthCtrlComboBox");

        gridLayout->addWidget(depthCtrlComboBox, 0, 1, 1, 1);

        label_2 = new QLabel(mainParametersGroupBox);
        label_2->setObjectName("label_2");

        gridLayout->addWidget(label_2, 1, 0, 1, 1);

        depthSpinBox = new QSpinBox(mainParametersGroupBox);
        depthSpinBox->setObjectName("depthSpinBox");
        depthSpinBox->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        depthSpinBox->setMaximum(300);

        gridLayout->addWidget(depthSpinBox, 1, 1, 1, 1);


        vboxLayout2->addWidget(mainParametersGroupBox);

        audioRoutingGroupBox = new QGroupBox(qsamplerChannelFxForm);
        audioRoutingGroupBox->setObjectName("audioRoutingGroupBox");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Maximum);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(audioRoutingGroupBox->sizePolicy().hasHeightForWidth());
        audioRoutingGroupBox->setSizePolicy(sizePolicy2);
        hboxLayout2 = new QHBoxLayout(audioRoutingGroupBox);
        hboxLayout2->setObjectName("hboxLayout2");
        audioRoutingTable = new QTableView(audioRoutingGroupBox);
        audioRoutingTable->setObjectName("audioRoutingTable");

        hboxLayout2->addWidget(audioRoutingTable);


        vboxLayout2->addWidget(audioRoutingGroupBox);


        hboxLayout->addLayout(vboxLayout2);


        vboxLayout->addLayout(hboxLayout);

        buttonBox = new QDialogButtonBox(qsamplerChannelFxForm);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Apply|QDialogButtonBox::Cancel|QDialogButtonBox::NoButton|QDialogButtonBox::Ok|QDialogButtonBox::Reset);

        vboxLayout->addWidget(buttonBox);


        retranslateUi(qsamplerChannelFxForm);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, qsamplerChannelFxForm, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, qsamplerChannelFxForm, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(qsamplerChannelFxForm);
    } // setupUi

    void retranslateUi(QDialog *qsamplerChannelFxForm)
    {
        qsamplerChannelFxForm->setWindowTitle(QCoreApplication::translate("qsamplerChannelFxForm", "Channel Effects", nullptr));
        groupBox->setTitle(QCoreApplication::translate("qsamplerChannelFxForm", "FX Send Selection", nullptr));
#if QT_CONFIG(tooltip)
        createPushButton->setToolTip(QCoreApplication::translate("qsamplerChannelFxForm", "Creates a new FX Send.\n"
"You have to select 'Apply' afterwards\n"
"to actually create it on sampler side.", nullptr));
#endif // QT_CONFIG(tooltip)
        createPushButton->setText(QCoreApplication::translate("qsamplerChannelFxForm", "Create", nullptr));
#if QT_CONFIG(tooltip)
        destroyPushButton->setToolTip(QCoreApplication::translate("qsamplerChannelFxForm", "Schedules the selected FX send for deletion.\n"
"You have to select 'Apply' afterwards to\n"
"actually destroy it on sampler side.", nullptr));
#endif // QT_CONFIG(tooltip)
        destroyPushButton->setText(QCoreApplication::translate("qsamplerChannelFxForm", "Destroy", nullptr));
        mainParametersGroupBox->setTitle(QCoreApplication::translate("qsamplerChannelFxForm", "FX Send's Parameters", nullptr));
        label->setText(QCoreApplication::translate("qsamplerChannelFxForm", "Send Depth\n"
"MIDI Controller:", nullptr));
        label_2->setText(QCoreApplication::translate("qsamplerChannelFxForm", "Current Depth:", nullptr));
        depthSpinBox->setSuffix(QCoreApplication::translate("qsamplerChannelFxForm", "%", nullptr));
        audioRoutingGroupBox->setTitle(QCoreApplication::translate("qsamplerChannelFxForm", "Audio Routing", nullptr));
    } // retranslateUi

};

namespace Ui {
    class qsamplerChannelFxForm: public Ui_qsamplerChannelFxForm {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_QSAMPLERCHANNELFXFORM_H
