/*
qsampler - A LinuxSampler Qt GUI Interface.

   Copyright (C) 2004-2021, rncbc aka Rui Nuno Capela. All rights reserved.
   Copyright (C) 2007, Christian Schoenebeck

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

 
*/

/********************************************************************************
** Form generated from reading UI file 'qsamplerInstrumentForm.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_QSAMPLERINSTRUMENTFORM_H
#define UI_QSAMPLERINSTRUMENTFORM_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_qsamplerInstrumentForm
{
public:
    QVBoxLayout *vboxLayout;
    QGridLayout *gridLayout;
    QComboBox *EngineNameComboBox;
    QLabel *EngineNameTextLabel;
    QLabel *ProgTextLabel;
    QSpinBox *ProgSpinBox;
    QLabel *VolumeTextLabel;
    QLabel *MapTextLabel;
    QLabel *BankTextLabel;
    QSpinBox *BankSpinBox;
    QComboBox *InstrumentFileComboBox;
    QLabel *LoadModeNameTextLabel;
    QSpacerItem *spacerItem;
    QLabel *InstrumentFileTextLabel;
    QLabel *NameTextLabe;
    QComboBox *MapComboBox;
    QLabel *InstrumentNrTextLabel;
    QComboBox *InstrumentNrComboBox;
    QSpinBox *VolumeSpinBox;
    QLineEdit *NameLineEdit;
    QComboBox *LoadModeComboBox;
    QToolButton *InstrumentFileToolButton;
    QDialogButtonBox *DialogButtonBox;

    void setupUi(QDialog *qsamplerInstrumentForm)
    {
        if (qsamplerInstrumentForm->objectName().isEmpty())
            qsamplerInstrumentForm->setObjectName("qsamplerInstrumentForm");
        qsamplerInstrumentForm->resize(452, 226);
        qsamplerInstrumentForm->setFocusPolicy(Qt::StrongFocus);
        const QIcon icon = QIcon(QString::fromUtf8(":/images/qsamplerInstrument.png"));
        qsamplerInstrumentForm->setWindowIcon(icon);
        vboxLayout = new QVBoxLayout(qsamplerInstrumentForm);
        vboxLayout->setSpacing(4);
        vboxLayout->setContentsMargins(4, 4, 4, 4);
        vboxLayout->setObjectName("vboxLayout");
        gridLayout = new QGridLayout();
        gridLayout->setSpacing(4);
        gridLayout->setContentsMargins(0, 0, 0, 0);
        gridLayout->setObjectName("gridLayout");
        EngineNameComboBox = new QComboBox(qsamplerInstrumentForm);
        EngineNameComboBox->setObjectName("EngineNameComboBox");

        gridLayout->addWidget(EngineNameComboBox, 2, 1, 1, 8);

        EngineNameTextLabel = new QLabel(qsamplerInstrumentForm);
        EngineNameTextLabel->setObjectName("EngineNameTextLabel");
        EngineNameTextLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        EngineNameTextLabel->setWordWrap(false);

        gridLayout->addWidget(EngineNameTextLabel, 2, 0, 1, 1);

        ProgTextLabel = new QLabel(qsamplerInstrumentForm);
        ProgTextLabel->setObjectName("ProgTextLabel");
        ProgTextLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        ProgTextLabel->setWordWrap(false);

        gridLayout->addWidget(ProgTextLabel, 0, 7, 1, 1);

        ProgSpinBox = new QSpinBox(qsamplerInstrumentForm);
        ProgSpinBox->setObjectName("ProgSpinBox");
        ProgSpinBox->setMaximum(128);
        ProgSpinBox->setMinimum(1);

        gridLayout->addWidget(ProgSpinBox, 0, 8, 1, 2);

        VolumeTextLabel = new QLabel(qsamplerInstrumentForm);
        VolumeTextLabel->setObjectName("VolumeTextLabel");
        VolumeTextLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        VolumeTextLabel->setWordWrap(false);

        gridLayout->addWidget(VolumeTextLabel, 5, 0, 1, 1);

        MapTextLabel = new QLabel(qsamplerInstrumentForm);
        MapTextLabel->setObjectName("MapTextLabel");
        MapTextLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        MapTextLabel->setWordWrap(false);

        gridLayout->addWidget(MapTextLabel, 0, 0, 1, 1);

        BankTextLabel = new QLabel(qsamplerInstrumentForm);
        BankTextLabel->setObjectName("BankTextLabel");
        BankTextLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        BankTextLabel->setWordWrap(false);

        gridLayout->addWidget(BankTextLabel, 0, 3, 1, 2);

        BankSpinBox = new QSpinBox(qsamplerInstrumentForm);
        BankSpinBox->setObjectName("BankSpinBox");
        BankSpinBox->setMaximum(16383);

        gridLayout->addWidget(BankSpinBox, 0, 5, 1, 2);

        InstrumentFileComboBox = new QComboBox(qsamplerInstrumentForm);
        InstrumentFileComboBox->setObjectName("InstrumentFileComboBox");
        InstrumentFileComboBox->setMinimumSize(QSize(320, 0));
        InstrumentFileComboBox->setEditable(true);

        gridLayout->addWidget(InstrumentFileComboBox, 3, 1, 1, 8);

        LoadModeNameTextLabel = new QLabel(qsamplerInstrumentForm);
        LoadModeNameTextLabel->setObjectName("LoadModeNameTextLabel");
        LoadModeNameTextLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        LoadModeNameTextLabel->setWordWrap(false);

        gridLayout->addWidget(LoadModeNameTextLabel, 5, 4, 1, 2);

        spacerItem = new QSpacerItem(119, 8, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout->addItem(spacerItem, 5, 2, 1, 2);

        InstrumentFileTextLabel = new QLabel(qsamplerInstrumentForm);
        InstrumentFileTextLabel->setObjectName("InstrumentFileTextLabel");
        InstrumentFileTextLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        InstrumentFileTextLabel->setWordWrap(false);

        gridLayout->addWidget(InstrumentFileTextLabel, 3, 0, 1, 1);

        NameTextLabe = new QLabel(qsamplerInstrumentForm);
        NameTextLabe->setObjectName("NameTextLabe");
        NameTextLabe->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        NameTextLabe->setWordWrap(false);

        gridLayout->addWidget(NameTextLabe, 1, 0, 1, 1);

        MapComboBox = new QComboBox(qsamplerInstrumentForm);
        MapComboBox->setObjectName("MapComboBox");

        gridLayout->addWidget(MapComboBox, 0, 1, 1, 2);

        InstrumentNrTextLabel = new QLabel(qsamplerInstrumentForm);
        InstrumentNrTextLabel->setObjectName("InstrumentNrTextLabel");
        InstrumentNrTextLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        InstrumentNrTextLabel->setWordWrap(false);

        gridLayout->addWidget(InstrumentNrTextLabel, 4, 0, 1, 1);

        InstrumentNrComboBox = new QComboBox(qsamplerInstrumentForm);
        InstrumentNrComboBox->setObjectName("InstrumentNrComboBox");
        InstrumentNrComboBox->setMinimumSize(QSize(320, 0));

        gridLayout->addWidget(InstrumentNrComboBox, 4, 1, 1, 9);

        VolumeSpinBox = new QSpinBox(qsamplerInstrumentForm);
        VolumeSpinBox->setObjectName("VolumeSpinBox");
        VolumeSpinBox->setMaximum(100);

        gridLayout->addWidget(VolumeSpinBox, 5, 1, 1, 1);

        NameLineEdit = new QLineEdit(qsamplerInstrumentForm);
        NameLineEdit->setObjectName("NameLineEdit");

        gridLayout->addWidget(NameLineEdit, 1, 1, 1, 9);

        LoadModeComboBox = new QComboBox(qsamplerInstrumentForm);
        LoadModeComboBox->addItem(QString());
        LoadModeComboBox->addItem(QString());
        LoadModeComboBox->addItem(QString());
        LoadModeComboBox->addItem(QString());
        LoadModeComboBox->setObjectName("LoadModeComboBox");

        gridLayout->addWidget(LoadModeComboBox, 5, 6, 1, 4);

        InstrumentFileToolButton = new QToolButton(qsamplerInstrumentForm);
        InstrumentFileToolButton->setObjectName("InstrumentFileToolButton");
        InstrumentFileToolButton->setMinimumSize(QSize(24, 24));
        InstrumentFileToolButton->setMaximumSize(QSize(26, 26));
        InstrumentFileToolButton->setFocusPolicy(Qt::TabFocus);
        const QIcon icon1 = QIcon(QString::fromUtf8(":/images/fileOpen.png"));
        InstrumentFileToolButton->setIcon(icon1);

        gridLayout->addWidget(InstrumentFileToolButton, 3, 9, 1, 1);


        vboxLayout->addLayout(gridLayout);

        DialogButtonBox = new QDialogButtonBox(qsamplerInstrumentForm);
        DialogButtonBox->setObjectName("DialogButtonBox");
        DialogButtonBox->setOrientation(Qt::Horizontal);
        DialogButtonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        vboxLayout->addWidget(DialogButtonBox);

#if QT_CONFIG(shortcut)
        EngineNameTextLabel->setBuddy(EngineNameComboBox);
        ProgTextLabel->setBuddy(ProgSpinBox);
        VolumeTextLabel->setBuddy(VolumeSpinBox);
        MapTextLabel->setBuddy(MapComboBox);
        BankTextLabel->setBuddy(BankSpinBox);
        LoadModeNameTextLabel->setBuddy(LoadModeComboBox);
        InstrumentFileTextLabel->setBuddy(InstrumentFileComboBox);
        NameTextLabe->setBuddy(NameLineEdit);
        InstrumentNrTextLabel->setBuddy(InstrumentNrComboBox);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(MapComboBox, BankSpinBox);
        QWidget::setTabOrder(BankSpinBox, ProgSpinBox);
        QWidget::setTabOrder(ProgSpinBox, NameLineEdit);
        QWidget::setTabOrder(NameLineEdit, EngineNameComboBox);
        QWidget::setTabOrder(EngineNameComboBox, InstrumentFileComboBox);
        QWidget::setTabOrder(InstrumentFileComboBox, InstrumentFileToolButton);
        QWidget::setTabOrder(InstrumentFileToolButton, InstrumentNrComboBox);
        QWidget::setTabOrder(InstrumentNrComboBox, VolumeSpinBox);
        QWidget::setTabOrder(VolumeSpinBox, LoadModeComboBox);
        QWidget::setTabOrder(LoadModeComboBox, DialogButtonBox);

        retranslateUi(qsamplerInstrumentForm);

        QMetaObject::connectSlotsByName(qsamplerInstrumentForm);
    } // setupUi

    void retranslateUi(QDialog *qsamplerInstrumentForm)
    {
        qsamplerInstrumentForm->setWindowTitle(QCoreApplication::translate("qsamplerInstrumentForm", "Instrument", nullptr));
#if QT_CONFIG(tooltip)
        EngineNameComboBox->setToolTip(QCoreApplication::translate("qsamplerInstrumentForm", "Engine name", nullptr));
#endif // QT_CONFIG(tooltip)
        EngineNameTextLabel->setText(QCoreApplication::translate("qsamplerInstrumentForm", "&Engine:", nullptr));
        ProgTextLabel->setText(QCoreApplication::translate("qsamplerInstrumentForm", "&Prog:", nullptr));
#if QT_CONFIG(tooltip)
        ProgSpinBox->setToolTip(QCoreApplication::translate("qsamplerInstrumentForm", "Program (0-127)", nullptr));
#endif // QT_CONFIG(tooltip)
        VolumeTextLabel->setText(QCoreApplication::translate("qsamplerInstrumentForm", "Vol&ume:", nullptr));
        MapTextLabel->setText(QCoreApplication::translate("qsamplerInstrumentForm", "&Map:", nullptr));
        BankTextLabel->setText(QCoreApplication::translate("qsamplerInstrumentForm", "&Bank:", nullptr));
#if QT_CONFIG(tooltip)
        BankSpinBox->setToolTip(QCoreApplication::translate("qsamplerInstrumentForm", "Bank (0-16383)", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        InstrumentFileComboBox->setToolTip(QCoreApplication::translate("qsamplerInstrumentForm", "Instrument filename", nullptr));
#endif // QT_CONFIG(tooltip)
        LoadModeNameTextLabel->setText(QCoreApplication::translate("qsamplerInstrumentForm", "M&ode:", nullptr));
        InstrumentFileTextLabel->setText(QCoreApplication::translate("qsamplerInstrumentForm", "&Filename:", nullptr));
        NameTextLabe->setText(QCoreApplication::translate("qsamplerInstrumentForm", "&Name:", nullptr));
#if QT_CONFIG(tooltip)
        MapComboBox->setToolTip(QCoreApplication::translate("qsamplerInstrumentForm", "Instrument map", nullptr));
#endif // QT_CONFIG(tooltip)
        InstrumentNrTextLabel->setText(QCoreApplication::translate("qsamplerInstrumentForm", "&Instrument:", nullptr));
#if QT_CONFIG(tooltip)
        InstrumentNrComboBox->setToolTip(QCoreApplication::translate("qsamplerInstrumentForm", "Instrument name", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        VolumeSpinBox->setToolTip(QCoreApplication::translate("qsamplerInstrumentForm", "Volume (%)", nullptr));
#endif // QT_CONFIG(tooltip)
        VolumeSpinBox->setSuffix(QCoreApplication::translate("qsamplerInstrumentForm", " %", nullptr));
#if QT_CONFIG(tooltip)
        NameLineEdit->setToolTip(QCoreApplication::translate("qsamplerInstrumentForm", "Name", nullptr));
#endif // QT_CONFIG(tooltip)
        LoadModeComboBox->setItemText(0, QCoreApplication::translate("qsamplerInstrumentForm", "Default", nullptr));
        LoadModeComboBox->setItemText(1, QCoreApplication::translate("qsamplerInstrumentForm", "On Demand", nullptr));
        LoadModeComboBox->setItemText(2, QCoreApplication::translate("qsamplerInstrumentForm", "On Demand Hold", nullptr));
        LoadModeComboBox->setItemText(3, QCoreApplication::translate("qsamplerInstrumentForm", "Persistent", nullptr));

#if QT_CONFIG(tooltip)
        LoadModeComboBox->setToolTip(QCoreApplication::translate("qsamplerInstrumentForm", "Load mode", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        InstrumentFileToolButton->setToolTip(QCoreApplication::translate("qsamplerInstrumentForm", "Browse for instrument filename", nullptr));
#endif // QT_CONFIG(tooltip)
        InstrumentFileToolButton->setText(QString());
    } // retranslateUi

};

namespace Ui {
    class qsamplerInstrumentForm: public Ui_qsamplerInstrumentForm {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_QSAMPLERINSTRUMENTFORM_H
