/****************************************************************************
** Meta object code from reading C++ file 'qsamplerMainForm.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../qsampler-1.0.2/src/qsamplerMainForm.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qsamplerMainForm.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN8QSampler8MainFormE_t {};
} // unnamed namespace

template <> constexpr inline auto QSampler::MainForm::qt_create_metaobjectdata<qt_meta_tag_ZN8QSampler8MainFormE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QSampler::MainForm",
        "fileNew",
        "",
        "fileOpen",
        "fileOpenRecent",
        "fileSave",
        "fileSaveAs",
        "fileReset",
        "fileRestart",
        "fileExit",
        "editAddChannel",
        "editRemoveChannel",
        "editSetupChannel",
        "editEditChannel",
        "editResetChannel",
        "editResetAllChannels",
        "viewMenubar",
        "bOn",
        "viewToolbar",
        "viewStatusbar",
        "viewMessages",
        "viewInstruments",
        "viewDevices",
        "viewOptions",
        "channelsArrange",
        "channelsAutoArrange",
        "helpAboutQt",
        "helpAbout",
        "stabilizeForm",
        "updateRecentFilesMenu",
        "volumeChanged",
        "iVolume",
        "channelStripChanged",
        "ChannelStrip*",
        "pChannelStrip",
        "channelsMenuAboutToShow",
        "channelsMenuActivated",
        "timerSlot",
        "readServerStdout",
        "processServerExit",
        "autoReconnectClient",
        "handle_sigusr1",
        "handle_sigterm",
        "activateStrip",
        "QMdiSubWindow*",
        "pMdiSubWindow",
        "channelsToolbarOrientation",
        "Qt::Orientation",
        "orientation"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'fileNew'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'fileOpen'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'fileOpenRecent'
        QtMocHelpers::SlotData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'fileSave'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'fileSaveAs'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'fileReset'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'fileRestart'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'fileExit'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'editAddChannel'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'editRemoveChannel'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'editSetupChannel'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'editEditChannel'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'editResetChannel'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'editResetAllChannels'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'viewMenubar'
        QtMocHelpers::SlotData<void(bool)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 17 },
        }}),
        // Slot 'viewToolbar'
        QtMocHelpers::SlotData<void(bool)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 17 },
        }}),
        // Slot 'viewStatusbar'
        QtMocHelpers::SlotData<void(bool)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 17 },
        }}),
        // Slot 'viewMessages'
        QtMocHelpers::SlotData<void(bool)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 17 },
        }}),
        // Slot 'viewInstruments'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'viewDevices'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'viewOptions'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'channelsArrange'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'channelsAutoArrange'
        QtMocHelpers::SlotData<void(bool)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 17 },
        }}),
        // Slot 'helpAboutQt'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'helpAbout'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'stabilizeForm'
        QtMocHelpers::SlotData<void()>(28, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'updateRecentFilesMenu'
        QtMocHelpers::SlotData<void()>(29, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'volumeChanged'
        QtMocHelpers::SlotData<void(int)>(30, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 31 },
        }}),
        // Slot 'channelStripChanged'
        QtMocHelpers::SlotData<void(ChannelStrip *)>(32, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 33, 34 },
        }}),
        // Slot 'channelsMenuAboutToShow'
        QtMocHelpers::SlotData<void()>(35, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'channelsMenuActivated'
        QtMocHelpers::SlotData<void()>(36, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'timerSlot'
        QtMocHelpers::SlotData<void()>(37, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'readServerStdout'
        QtMocHelpers::SlotData<void()>(38, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'processServerExit'
        QtMocHelpers::SlotData<void()>(39, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'autoReconnectClient'
        QtMocHelpers::SlotData<void()>(40, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'handle_sigusr1'
        QtMocHelpers::SlotData<void()>(41, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'handle_sigterm'
        QtMocHelpers::SlotData<void()>(42, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'activateStrip'
        QtMocHelpers::SlotData<void(QMdiSubWindow *)>(43, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 44, 45 },
        }}),
        // Slot 'channelsToolbarOrientation'
        QtMocHelpers::SlotData<void(Qt::Orientation)>(46, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 47, 48 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<MainForm, qt_meta_tag_ZN8QSampler8MainFormE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QSampler::MainForm::staticMetaObject = { {
    QMetaObject::SuperData::link<QMainWindow::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8QSampler8MainFormE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8QSampler8MainFormE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8QSampler8MainFormE_t>.metaTypes,
    nullptr
} };

void QSampler::MainForm::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<MainForm *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->fileNew(); break;
        case 1: _t->fileOpen(); break;
        case 2: _t->fileOpenRecent(); break;
        case 3: _t->fileSave(); break;
        case 4: _t->fileSaveAs(); break;
        case 5: _t->fileReset(); break;
        case 6: _t->fileRestart(); break;
        case 7: _t->fileExit(); break;
        case 8: _t->editAddChannel(); break;
        case 9: _t->editRemoveChannel(); break;
        case 10: _t->editSetupChannel(); break;
        case 11: _t->editEditChannel(); break;
        case 12: _t->editResetChannel(); break;
        case 13: _t->editResetAllChannels(); break;
        case 14: _t->viewMenubar((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 15: _t->viewToolbar((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 16: _t->viewStatusbar((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 17: _t->viewMessages((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 18: _t->viewInstruments(); break;
        case 19: _t->viewDevices(); break;
        case 20: _t->viewOptions(); break;
        case 21: _t->channelsArrange(); break;
        case 22: _t->channelsAutoArrange((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 23: _t->helpAboutQt(); break;
        case 24: _t->helpAbout(); break;
        case 25: _t->stabilizeForm(); break;
        case 26: _t->updateRecentFilesMenu(); break;
        case 27: _t->volumeChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 28: _t->channelStripChanged((*reinterpret_cast<std::add_pointer_t<ChannelStrip*>>(_a[1]))); break;
        case 29: _t->channelsMenuAboutToShow(); break;
        case 30: _t->channelsMenuActivated(); break;
        case 31: _t->timerSlot(); break;
        case 32: _t->readServerStdout(); break;
        case 33: _t->processServerExit(); break;
        case 34: _t->autoReconnectClient(); break;
        case 35: _t->handle_sigusr1(); break;
        case 36: _t->handle_sigterm(); break;
        case 37: _t->activateStrip((*reinterpret_cast<std::add_pointer_t<QMdiSubWindow*>>(_a[1]))); break;
        case 38: _t->channelsToolbarOrientation((*reinterpret_cast<std::add_pointer_t<Qt::Orientation>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *QSampler::MainForm::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QSampler::MainForm::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8QSampler8MainFormE_t>.strings))
        return static_cast<void*>(this);
    return QMainWindow::qt_metacast(_clname);
}

int QSampler::MainForm::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QMainWindow::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 39)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 39;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 39)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 39;
    }
    return _id;
}
QT_WARNING_POP
